﻿Module ODFcode
    ' ENSEMBLE DES SUB, VARIABLES ET STRUCTURES POUR CONSTRUIRE LE CODF

    Structure knobStructure
        Public used As Boolean ' indique si utilisé
        Public isRank As Boolean
        Public isPedalRank As Boolean
        Public isCoupler As Boolean
        Public isTremulant As Boolean ' le bouton active le tremulant
        Public isTremulated As Boolean ' le jeu est tremulable
        Public isEnclosed As Boolean
        Public manual As Integer ' indique le clavier d'apartenance
        Public stopcode As Integer ' 2000/2099 pedalier, 2100/2199 great, 2200/2299 manual2 etc
        Public tremulantCode As Integer
        Public enclosureCode As Integer
        Public couplerCode As Integer
        Public w As Integer ' width
        Public h As Integer ' height
        Dim size As System.Drawing.Size
        Public loc1 As System.Drawing.Point ' location dans console
        Public loc2 As System.Drawing.Point ' location dans jamb
        Public jamb As Single
        Public color As System.Drawing.Color
        Public name As String ' nom affiché
        Public numberOfPipes As Integer
        Public folder As String ' nom du fichier dans 001512
        Public subFolder As String ' nom du dossier parent de folder : manuals ou pedals
        Public transpo As Integer ' transposition si jeux emprunté
    End Structure
    Public knob(96) As knobStructure
    Structure divisionStructure
        Public code As Integer
        Public name As String
        Public shortName As String
        Public numberOfKeys As Integer
        Public displayPage As Integer
        Public styleID As Integer
        Public X As Integer
        Public Y As Integer
        Public volume As Single
    End Structure
    Public division(6) As divisionStructure
    Structure couplerStructure
        Public couplerCode As Integer
        Public name As String
        Public styleID1 As Integer
        Public x1 As Integer
        Public y1 As Integer
        Public styleID2 As Integer
        Public x2 As Integer
        Public y2 As Integer
        Public jamb As Integer
        Public knobNumber As String ' pour recharger CODF
        Public KnobName As String  ' pour recharger CODF

    End Structure
    Public coupler(96) As couplerStructure
    Structure tremulantStructure
        Public tremulantCode As Integer
        Public name As String
        Public manual As Integer ' indique le clavier d'apartenance
        Public styleID1 As Integer
        Public x1 As Integer
        Public y1 As Integer
        Public styleID2 As Integer
        Public x2 As Integer
        Public y2 As Integer
        Public jamb As Integer
    End Structure
    Public tremulant(96) As tremulantStructure
    Structure labelStructure
        Public isUsed As Boolean
        Public page As Integer
        Public name As String
        Public text As String
        Public posX As Integer
        Public posY As Integer
    End Structure
    Public label(20) As labelStructure
    Structure bandehStructure
        Public posX As Single
        Public posY As Single
        Public jamb As Single
    End Structure
    Public bandeH(4) As bandehStructure
    Structure swellBoxStructure
        Public enclosureCode As Integer
        Public name As String
        Public x1 As Integer
        Public y1 As Integer
    End Structure
    Public enclosure(6) As swellBoxStructure
    Structure rankStructure
        Public ID As Integer
        Public name As String
        Public manual As Integer
        Public numberOfPipes As Integer
        Public folder As String
        Public hasTremulant As Boolean
        Public tremulantCode As Integer
        Public isEnclosed As Boolean
        Public enclosureCode As Integer
        Public knobNumber As String ' pour recharger CODF
        Public KnobName As String  ' pour recharger CODF
        Public knobReloadCode As String ' R pour rank, P pour pedalRank
        Public vol36 As Single
        Public vol96 As Single
    End Structure
    Public rank(96) As rankStructure
    Structure stoprankStructure
        Public name As String
        Public stopCode As Integer
        Public rankID As Integer
        Public numberOfKeys As Integer
    End Structure
    Public stopRank(96) As stoprankStructure
    Structure stopStructure
        Public stopCode As Integer
        Public name As String
        Public displayname As String
        Public text As String
        Public styleID1 As Integer
        Public x1 As Integer
        Public y1 As Integer
        Public styleID2 As Integer
        Public x2 As Integer
        Public y2 As Integer
        Public jamb As Integer
    End Structure
    Public stopknob(96) As stopStructure
    Structure manualStructure
        Public name As String
        Public isEnclosed As Boolean
        Public nbrOfKeys As Integer
        Public X1 As Integer
        Public Y1 As Integer
    End Structure
    Public manual(6) As manualStructure

    ' VARIABLES
    Public nbrOfManuals As Integer = 5 ' modifiable en page 3
    Public buttonOn As String = "yellow90x30_on.bmp"
    Public buttonOff As String = "yellow90x30_off.bmp"
    Public buttonID As Integer = 5
    Public jambButtonOn As String = "knob72on.bmp"
    Public jambButtonOff As String = "knob72off.bmp"
    Public jambButtonMask As String = "knob72mask.bmp"
    Public jambButtonID As Integer = 6
    Public needJamb2 As Boolean = False
    Public PboxImageOn As System.Drawing.Image
    Public PboxImageOff As System.Drawing.Image
    Public knobFont As System.Drawing.Font
    Public jambFont As System.Drawing.Font
    Public maxFlow As Integer ' wind fluctuation
    Public nbrOfLabels As Integer = 0
    Public tremulantFrequency As Single = 5.2

    ' HEADER & GENERAL
    Public UniqueOrganID As String = "891512"
    Public organName As String = "Basilica Organ"
    Public ShortName As String = "Basilica"
    Public OrganDefinitionFilename As String = "Basilica (891512)"
    ' REVERB
    Public reverbMode As Integer = 1 ' 1= no truncation
    Public reverbTime As Integer = 0 ' ideal 130 à 180 quand mode =2

    'PREPARER LES ELEMENTS DU CODF
    Public Sub prepare_couplers()
        ' BALAYER TOUS LES KNOBS A LA RECHERCHE DE COUPLERS
        Dim idx
        Dim counter As Integer = 0
        For idx = 0 To 96
            With knob(idx)
                If .isCoupler = True Then
                    counter += 1
                    coupler(idx).name = .name
                    coupler(idx).couplerCode = Main.getCouplerCode(.name)
                    coupler(idx).styleID1 = buttonID
                    coupler(idx).x1 = .loc1.X
                    coupler(idx).y1 = .loc1.Y
                    '
                    coupler(idx).styleID2 = jambButtonID
                    coupler(idx).jamb = .jamb
                    coupler(idx).x2 = .loc2.X
                    coupler(idx).y2 = .loc2.Y
                 '
                    coupler(idx).knobNumber = "#ID " & idx.ToString
                    coupler(idx).KnobName = .name

                    make_coupler(idx)
                    Main.LabelCodfInfo.Text = "Writing Coupler " & idx.ToString : Main.LabelCodfInfo.Refresh()
                End If
            End With
        Next
    End Sub
    Public Sub prepare_tremulants()
        ' BALAYER TOUS LES KNOBS A LA RECHERCHE DE TREMULANTS
        Dim idx
        Dim counter As Integer = 0
        For idx = 0 To 96
            With knob(idx)
                If .isTremulant = True Then
                    counter += 1
                    tremulant(idx).name = .name
                    tremulant(idx).manual = knob(idx).manual
                    tremulant(idx).tremulantCode = Main.GetTremulantCode(.manual)
                    tremulant(idx).styleID1 = buttonID
                    tremulant(idx).x1 = .loc1.X
                    tremulant(idx).y1 = .loc1.Y
                    '
                    tremulant(idx).styleID2 = jambButtonID
                    tremulant(idx).jamb = .jamb
                    tremulant(idx).x2 = .loc2.X
                    tremulant(idx).y2 = .loc2.Y
                  '
                    make_tremulant(idx)
                    Main.LabelCodfInfo.Text = "Writing tremulant " & idx.ToString : Main.LabelCodfInfo.Refresh()
                End If
            End With
        Next
    End Sub
    Public Sub prepare_enclosures()
        ' BALAYER TOUS LES KNOBS A LA RECHERCHE DE RANKS ENCLOSED

        Main.LabelCodfInfo.Text = "Preparing Enclosures " : Main.LabelCodfInfo.Refresh()

        manual(5).isEnclosed = False
        manual(4).isEnclosed = False
        manual(3).isEnclosed = False
        manual(2).isEnclosed = False
        manual(1).isEnclosed = False
        Dim idx
        For idx = 1 To 96
            With knob(idx)
                If .isRank = True Then
                    If .manual = 5 And Main.CheckBoxSwell_V.Checked = True Then
                        .isEnclosed = True
                        manual(5).isEnclosed = True
                        .enclosureCode = Main.getEnclosureCode(.manual)
                        enclosure(.manual).enclosureCode = .enclosureCode
                        enclosure(.manual).x1 = 940 : enclosure(.manual).y1 = 20
                    End If
                    If .manual = 4 And Main.CheckBoxSwell_IV.Checked = True Then
                        manual(4).isEnclosed = True
                        .isEnclosed = True
                        enclosure(.manual).enclosureCode = Main.getEnclosureCode(.manual)
                        enclosure(.manual).x1 = 940 : enclosure(.manual).y1 = 110
                    End If
                    If .manual = 3 And Main.CheckBoxSwell_III.Checked = True Then
                        manual(3).isEnclosed = True
                        .isEnclosed = True
                        enclosure(.manual).enclosureCode = Main.getEnclosureCode(.manual)
                        enclosure(.manual).x1 = 940 : enclosure(.manual).y1 = 200
                    End If
                    If .manual = 2 And Main.CheckBoxSwell_II.Checked = True Then
                        manual(2).isEnclosed = True
                        .isEnclosed = True
                        enclosure(.manual).enclosureCode = Main.getEnclosureCode(.manual)
                        enclosure(.manual).x1 = 940 : enclosure(.manual).y1 = 290
                    End If
                    If .manual = 1 And Main.CheckBoxSwell_I.Checked = True Then
                        manual(1).isEnclosed = True
                        .isEnclosed = True
                        enclosure(.manual).enclosureCode = Main.getEnclosureCode(.manual)
                        enclosure(.manual).x1 = 940 : enclosure(.manual).y1 = 380
                    End If
                    enclosure(.manual).name = "Enclosure Manual " & .manual.ToString
                End If
            End With

        Next
        If manual(5).isEnclosed = True Then make_enclosure(5)
        If manual(4).isEnclosed = True Then make_enclosure(4)
        If manual(3).isEnclosed = True Then make_enclosure(3)
        If manual(2).isEnclosed = True Then make_enclosure(2)
        If manual(1).isEnclosed = True Then make_enclosure(1)
    End Sub
    Public Sub prepare_ranks()
        ' BALAYER TOUS LES KNOBS A LA RECHERCHE DE RANKS
        Dim idx
        Dim counter As Integer = 0
        For idx = 1 To 96
            With knob(idx)
                If .isRank = True Then
                    counter += 1
                    rank(idx).ID = counter
                    rank(idx).name = counter.ToString & " " & .name
                    rank(idx).folder = .subFolder & "/" & .folder
                    rank(idx).manual = .manual
                    rank(idx).numberOfPipes = .numberOfPipes
                    rank(idx).hasTremulant = .isTremulated
                    rank(idx).tremulantCode = .tremulantCode
                    rank(idx).isEnclosed = .isEnclosed
                    rank(idx).enclosureCode = .enclosureCode
                    rank(idx).knobNumber = "#ID " & idx.ToString
                    rank(idx).KnobName = .name
                    rank(idx).knobReloadCode = IIf(.isPedalRank = True, "P", "R")
                    rank(idx).vol36 = division(.manual).volume
                    rank(idx).vol96 = division(.manual).volume
                    make_rank(idx)
                    Main.LabelCodfInfo.Text = "Writing Rank " & idx.ToString : Main.LabelCodfInfo.Refresh()
                End If
            End With
        Next
    End Sub
    Public Sub prepare_stopRanks()
        ' BALAYER TOUS LES KNOBS A LA RECHERCHE DE RANKS ET CREER LES STOPRANKS
        Dim idx
        Dim counter As Integer = 0
        For idx = 0 To 96
            With knob(idx)
                If .isRank = True Then
                    counter += 1
                    stopRank(idx).name = counter.ToString & " " & .name
                    stopRank(idx).stopCode = knob(idx).stopcode
                    stopRank(idx).rankID = counter
                    stopRank(idx).numberOfKeys = 61 ' provisoire
                    make_stoprank(idx)
                    Main.LabelCodfInfo.Text = "Writing SopRank " & idx.ToString : Main.LabelCodfInfo.Refresh()
                End If
            End With
        Next
    End Sub
    Public Sub prepare_stops()
        ' BALAYER TOUS LES KNOBS A LA RECHERCHE DE RANKS ET CREER LES STOPS
        Dim idx
        Dim counter As Integer = 0

        For idx = 1 To 96
            With knob(idx)
                If .isRank = True Then
                    counter += 1
                    stopknob(idx).name = counter.ToString & " " & .name
                    stopknob(idx).displayname = counter.ToString & " " & .name
                    stopknob(idx).stopCode = .stopcode
                    stopknob(idx).text = .name
                    ' retirer le # devant le nom des jeux rajoutés
                    If .name.Substring(0, 1) = "#" Then
                        stopknob(idx).text = .name.Substring(1, Len(.name) - 1)
                    End If
                    stopknob(idx).styleID1 = buttonID
                    stopknob(idx).x1 = .loc1.X
                    stopknob(idx).y1 = .loc1.Y
                    '
                    stopknob(idx).styleID2 = jambButtonID
                    stopknob(idx).jamb = .jamb
                    stopknob(idx).x2 = .loc2.X
                    stopknob(idx).y2 = .loc2.Y
                    '
                    make_Stop(idx)
                    Main.LabelCodfInfo.Text = "Writing Stop " & idx.ToString : Main.LabelCodfInfo.Refresh()
                End If
            End With
        Next
    End Sub
    Public Sub prepare_labels()
        Dim idx
        For idx = 0 To UBound(label)
            If label(idx).isUsed = True Then
                make_labels(idx)
            End If
        Next
    End Sub

    ' CONSTRUIRE LE CODF
    Public Sub make_Header()
        Main.RtbCODF.Text = ""
        Main.RtbCODF.Text += "<?xml version=" & Chr(34) & "1.0" & Chr(34) & " encoding=" & Chr(34) & "UTF-8" & Chr(34) & "?>" & vbCrLf
        Main.RtbCODF.Text += "<Hauptwerk FileFormat = " & Chr(34) & "CustomOrgan" & Chr(34) & " FileFormatVersion = " & Chr(34) & "4.0" & Chr(34) & ">" & vbCrLf
        Main.RtbCODF.Text += "<!-- Basilica version " & version & " -->" & vbCrLf
        Main.RtbCODF.Text += "<!-- " & nbrOfManuals & " manuals -->" & vbCrLf
        Main.RtbCODF.Text += "<!-- Volume  " & division(0).volume & " -->" & vbCrLf
        Main.RtbCODF.Text += "<!-- Volume  " & division(1).volume & " -->" & vbCrLf
        Main.RtbCODF.Text += "<!-- Volume  " & division(2).volume & " -->" & vbCrLf
        Main.RtbCODF.Text += "<!-- Volume  " & division(3).volume & " -->" & vbCrLf
        Main.RtbCODF.Text += "<!-- Volume  " & division(4).volume & " -->" & vbCrLf
        Main.RtbCODF.Text += "<!-- Volume  " & division(5).volume & " -->" & vbCrLf

        Main.RtbCODF.Text += "<!-- copyright : Copyright © VPX 2014-->" & vbCrLf
        Main.RtbCODF.Text += vbCrLf
    End Sub
    Public Sub make_General()
        Main.RtbCODF.Text += "<ObjectList ObjectType = " & Chr(34) & "_general" & Chr(34) & ">" & vbCrLf
        Main.RtbCODF.Text += "<_general>" & vbCrLf
        Main.RtbCODF.Text += "<Sys_ObjectID>1</Sys_ObjectID>" & vbCrLf
        Main.RtbCODF.Text += "<UniqueOrganID>" & UniqueOrganID & "</UniqueOrganID>" & vbCrLf
        Main.RtbCODF.Text += "<Name>" & organName & "</Name>" & vbCrLf
        Main.RtbCODF.Text += "<ShortName>" & ShortName & "</ShortName>" & vbCrLf
        Main.RtbCODF.Text += "<OrganDefinitionFilenameExcludingExtension>" & OrganDefinitionFilename & "</OrganDefinitionFilenameExcludingExtension>" & vbCrLf
        Main.RtbCODF.Text += "<CurrentHauptwerkVersion>4</CurrentHauptwerkVersion>" & vbCrLf
        Main.RtbCODF.Text += "<SampleRateCode>2</SampleRateCode>" & vbCrLf
        Main.RtbCODF.Text += "<VisualAppearanceCode>1</VisualAppearanceCode>" & vbCrLf
        Main.RtbCODF.Text += "<CustomOrganInfo_InstallationPackageID>001512</CustomOrganInfo_InstallationPackageID>" & vbCrLf
        Main.RtbCODF.Text += "<CustomOrganInfo_InfoFilename>infos.pdf</CustomOrganInfo_InfoFilename>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay_ConsoleWidthPixels>1024</CustomDisplay_ConsoleWidthPixels>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay_ConsoleHeightPixels>680</CustomDisplay_ConsoleHeightPixels>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay_EnableStdPages_Controls>" & IIf(Main.CheckBoxControls.Checked = True, "Y", "N") & "</CustomDisplay_EnableStdPages_Controls>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay_EnableStdPages_Keyboards>" & IIf(Main.CheckBoxKeyboards.Checked = True, "Y", "N") & "</CustomDisplay_EnableStdPages_Keyboards>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay_EnableStdPages_Wind>" & IIf(Main.CheckBoxWinds.Checked = True, "Y", "N") & "</CustomDisplay_EnableStdPages_Wind>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay_EnableStdPages_Enclosures>" & IIf(Main.CheckBoxEnclosures.Checked = True, "Y", "N") & "</CustomDisplay_EnableStdPages_Enclosures>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay_DefaultDisplayPageID>1</CustomDisplay_DefaultDisplayPageID>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay1_Generals_NumberDisplayed></CustomDisplay1_Generals_NumberDisplayed>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay1_Generals_DisplayPageID></CustomDisplay1_Generals_DisplayPageID>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay1_Generals_ControlStyleID></CustomDisplay1_Generals_ControlStyleID>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay1_Generals_FirstDisplayedPistonPos_X></CustomDisplay1_Generals_FirstDisplayedPistonPos_X>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay1_Generals_FirstDisplayedPistonPos_Y></CustomDisplay1_Generals_FirstDisplayedPistonPos_Y>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay1_Generals_LastDisplayedPistonPos_X></CustomDisplay1_Generals_LastDisplayedPistonPos_X>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay1_Generals_LastDisplayedPistonPos_Y></CustomDisplay1_Generals_LastDisplayedPistonPos_Y>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay1_GeneralCancel_DisplayPageID></CustomDisplay1_GeneralCancel_DisplayPageID>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay1_GeneralCancel_Text></CustomDisplay1_GeneralCancel_Text>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay1_GeneralCancel_ControlStyleID></CustomDisplay1_GeneralCancel_ControlStyleID>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay1_GeneralCancel_PosPixels_X></CustomDisplay1_GeneralCancel_PosPixels_X>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay1_GeneralCancel_PosPixels_Y></CustomDisplay1_GeneralCancel_PosPixels_Y>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay1_SetterSwitch_DisplayPageID></CustomDisplay1_SetterSwitch_DisplayPageID>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay1_SetterSwitch_Text></CustomDisplay1_SetterSwitch_Text>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay1_SetterSwitch_ControlStyleID></CustomDisplay1_SetterSwitch_ControlStyleID>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay1_SetterSwitch_PosPixels_X></CustomDisplay1_SetterSwitch_PosPixels_X>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay1_SetterSwitch_PosPixels_Y></CustomDisplay1_SetterSwitch_PosPixels_Y>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay2_Generals_DisplayPageID></CustomDisplay2_Generals_DisplayPageID>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay2_Generals_NumberDisplayed>0</CustomDisplay2_Generals_NumberDisplayed>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay2_Generals_ControlStyleID></CustomDisplay2_Generals_ControlStyleID>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay2_Generals_FirstDisplayedPistonPos_X>0</CustomDisplay2_Generals_FirstDisplayedPistonPos_X>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay2_Generals_FirstDisplayedPistonPos_Y>0</CustomDisplay2_Generals_FirstDisplayedPistonPos_Y>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay2_Generals_LastDisplayedPistonPos_X>0</CustomDisplay2_Generals_LastDisplayedPistonPos_X>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay2_Generals_LastDisplayedPistonPos_Y>0</CustomDisplay2_Generals_LastDisplayedPistonPos_Y>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay2_GeneralCancel_DisplayPageID></CustomDisplay2_GeneralCancel_DisplayPageID>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay2_GeneralCancel_Text></CustomDisplay2_GeneralCancel_Text>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay2_GeneralCancel_ControlStyleID></CustomDisplay2_GeneralCancel_ControlStyleID>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay2_GeneralCancel_PosPixels_X>0</CustomDisplay2_GeneralCancel_PosPixels_X>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay2_GeneralCancel_PosPixels_Y>0</CustomDisplay2_GeneralCancel_PosPixels_Y>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay2_SetterSwitch_DisplayPageID></CustomDisplay2_SetterSwitch_DisplayPageID>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay2_SetterSwitch_Text></CustomDisplay2_SetterSwitch_Text>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay2_SetterSwitch_ControlStyleID></CustomDisplay2_SetterSwitch_ControlStyleID>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay2_SetterSwitch_PosPixels_X>0</CustomDisplay2_SetterSwitch_PosPixels_X>" & vbCrLf
        Main.RtbCODF.Text += "<CustomDisplay2_SetterSwitch_PosPixels_Y>0</CustomDisplay2_SetterSwitch_PosPixels_Y>" & vbCrLf
        Main.RtbCODF.Text += "</_general>" & vbCrLf
        Main.RtbCODF.Text += "</ObjectList>" & vbCrLf
        Main.RtbCODF.Text += vbCrLf
    End Sub
    Public Sub make_required()
        Main.RtbCODF.Text += "<ObjectList ObjectType = " & Chr(34) & "requiredinstallationpackage" & Chr(34) & ">" & vbCrLf
        Main.RtbCODF.Text += "<requiredinstallationpackage>" & vbCrLf
        Main.RtbCODF.Text += "<InstallationPackageID>001512</InstallationPackageID>" & vbCrLf
        Main.RtbCODF.Text += "<Name>BASILICA</Name>" & vbCrLf
        Main.RtbCODF.Text += "<ShortName>Basilica</ShortName>" & vbCrLf
        Main.RtbCODF.Text += "<PackageSupplierID>000051</PackageSupplierID>" & vbCrLf
        Main.RtbCODF.Text += "<SupplierName>Virtual Pipes Xport</SupplierName>" & vbCrLf
        Main.RtbCODF.Text += "<MinimumPackageVersion>1.00</MinimumPackageVersion>" & vbCrLf
        Main.RtbCODF.Text += "</requiredinstallationpackage>" & vbCrLf
        Main.RtbCODF.Text += "</ObjectList>" & vbCrLf
        Main.RtbCODF.Text += vbCrLf
    End Sub
    Public Sub make_CustomDisplayPage()

        Main.RtbCODF.Text += "<ObjectList ObjectType=" & Chr(34) & "customdisplaypage" & Chr(34) & ">" & vbCrLf

        ' TITTLE PAGE
        Main.RtbCODF.Text += "<customdisplaypage>" & vbCrLf
        Main.RtbCODF.Text += "<DisplayPageID>4</DisplayPageID>" & vbCrLf
        Main.RtbCODF.Text += "<Name>BASILICA</Name>" & vbCrLf
        Main.RtbCODF.Text += "<BackgroundImageFilename>Images/Basilica.png</BackgroundImageFilename>" & vbCrLf
        Main.RtbCODF.Text += "<BackgroundImageFileInstallationPackageID>001512</BackgroundImageFileInstallationPackageID>" & vbCrLf
        Main.RtbCODF.Text += "</customdisplaypage>" & vbCrLf

        ' MAIN PAGE
        Main.RtbCODF.Text += "<customdisplaypage>" & vbCrLf
        Main.RtbCODF.Text += "<DisplayPageID>1</DisplayPageID>" & vbCrLf
        Main.RtbCODF.Text += "<Name>Console</Name>" & vbCrLf
        Main.RtbCODF.Text += "<BackgroundImageFilename>Images/Background-001.png</BackgroundImageFilename>" & vbCrLf
        Main.RtbCODF.Text += "<BackgroundImageFileInstallationPackageID>001512</BackgroundImageFileInstallationPackageID>" & vbCrLf
        Main.RtbCODF.Text += "</customdisplaypage>" & vbCrLf

        ' LEFT JAMB
        Main.RtbCODF.Text += vbCrLf
        Main.RtbCODF.Text += "<customdisplaypage>" & vbCrLf
        Main.RtbCODF.Text += "<DisplayPageID>2</DisplayPageID>" & vbCrLf
        Main.RtbCODF.Text += "<Name>Left Jamb</Name>" & vbCrLf
        Main.RtbCODF.Text += "<BackgroundImageFilename>Images/Background-001.png</BackgroundImageFilename>" & vbCrLf
        Main.RtbCODF.Text += "<BackgroundImageFileInstallationPackageID>001512</BackgroundImageFileInstallationPackageID>" & vbCrLf
        Main.RtbCODF.Text += "</customdisplaypage>" & vbCrLf

        ' RIGHT JAMB
        If needJamb2 = True Then
            Main.RtbCODF.Text += vbCrLf
            Main.RtbCODF.Text += "<customdisplaypage>" & vbCrLf
            Main.RtbCODF.Text += "<DisplayPageID>3</DisplayPageID>" & vbCrLf
            Main.RtbCODF.Text += "<Name>Right Jamb</Name>" & vbCrLf
            Main.RtbCODF.Text += "<BackgroundImageFilename>Images/Background-001.png</BackgroundImageFilename>" & vbCrLf
            Main.RtbCODF.Text += "<BackgroundImageFileInstallationPackageID>001512</BackgroundImageFileInstallationPackageID>" & vbCrLf
            Main.RtbCODF.Text += "</customdisplaypage>" & vbCrLf
        End If

        ' END OF SECTION
        Main.RtbCODF.Text += "</ObjectList>" & vbCrLf
        Main.RtbCODF.Text += vbCrLf
    End Sub
    Public Sub make_Division(ByVal div As Integer)
        Main.LabelCodfInfo.Text = "Writing Divisions" : Main.LabelCodfInfo.Refresh()
        Dim txt As String = ""
        With division(div)
            txt += "<division>" & vbCrLf
            txt += "<DivisionCode>" & .code & "</DivisionCode>" & vbCrLf
            txt += "<Name>" & .name & "</Name>" & vbCrLf
            txt += "<ShortName>" & .shortName & "</ShortName>" & vbCrLf
            txt += "<NumberOfKeys>" & .numberOfKeys & "</NumberOfKeys>" & vbCrLf
            txt += "<MIDINoteNumOfFirstKey>36</MIDINoteNumOfFirstKey>" & vbCrLf
            txt += "<NumberOfKeyActionNodesIfDifferentFromNumberOfKeys>0</NumberOfKeyActionNodesIfDifferentFromNumberOfKeys>" & vbCrLf
            txt += "<MIDINoteNumOfFirstKeyActionNodeIfDifferentFromFirstKey>0</MIDINoteNumOfFirstKeyActionNodeIfDifferentFromFirstKey>" & vbCrLf
            txt += "<AccessibleForMIDIInputAndOutput>Y</AccessibleForMIDIInputAndOutput>" & vbCrLf
            txt += "<WindModel_EnableWindModel>Y</WindModel_EnableWindModel>" & vbCrLf
            txt += "<WindModel_TotalResrvrAndWndchstVolMetresCubed>2</WindModel_TotalResrvrAndWndchstVolMetresCubed>" & vbCrLf
            txt += "<WindModel_AreaOfReservoirTableMetresSquared>1.8</WindModel_AreaOfReservoirTableMetresSquared>" & vbCrLf
            txt += "<WindModel_ResrvrBellowsPositiveDampingCoeff>25</WindModel_ResrvrBellowsPositiveDampingCoeff>" & vbCrLf
            txt += "<WindModel_WindchestPressureDropPctAtMaxLoad>2</WindModel_WindchestPressureDropPctAtMaxLoad>" & vbCrLf
            txt += "<PercussiveEngagingKeyActionNoise_RankID></PercussiveEngagingKeyActionNoise_RankID>" & vbCrLf
            txt += "<PercussiveEngagingKeyActionNoise_ChooseMIDINoteNumRandomly>N</PercussiveEngagingKeyActionNoise_ChooseMIDINoteNumRandomly>" & vbCrLf
            txt += "<PercussiveDisengagingKeyActionNoise_RankID></PercussiveDisengagingKeyActionNoise_RankID>" & vbCrLf
            txt += "<PercussiveDisengagingKeyActionNoise_ChooseMIDINoteNumRandomly>N</PercussiveDisengagingKeyActionNoise_ChooseMIDINoteNumRandomly>" & vbCrLf
            txt += "<SustainingKeyActionNoise_RankID></SustainingKeyActionNoise_RankID>" & vbCrLf
            txt += "<SustainingKeyActionNoise_ChooseMIDINoteNumberRandomly>N</SustainingKeyActionNoise_ChooseMIDINoteNumberRandomly>" & vbCrLf
            txt += "<CustomDisplay1_Divisionals_DisplayPageID></CustomDisplay1_Divisionals_DisplayPageID>" & vbCrLf
            txt += "<CustomDisplay1_Divisionals_NumberDisplayed></CustomDisplay1_Divisionals_NumberDisplayed>" & vbCrLf
            txt += "<CustomDisplay1_Divisionals_ControlStyleID></CustomDisplay1_Divisionals_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay1_Divisionals_FirstDisplayedPistonPos_X></CustomDisplay1_Divisionals_FirstDisplayedPistonPos_X>" & vbCrLf
            txt += "<CustomDisplay1_Divisionals_FirstDisplayedPistonPos_Y></CustomDisplay1_Divisionals_FirstDisplayedPistonPos_Y>" & vbCrLf
            txt += "<CustomDisplay1_Divisionals_LastDisplayedPistonPos_X></CustomDisplay1_Divisionals_LastDisplayedPistonPos_X>" & vbCrLf
            txt += "<CustomDisplay1_Divisionals_LastDisplayedPistonPos_Y></CustomDisplay1_Divisionals_LastDisplayedPistonPos_Y>" & vbCrLf
            txt += "<CustomDisplay1_DivisionalCancel_DisplayPageID></CustomDisplay1_DivisionalCancel_DisplayPageID>" & vbCrLf
            txt += "<CustomDisplay1_DivisionalCancel_Text></CustomDisplay1_DivisionalCancel_Text>" & vbCrLf
            txt += "<CustomDisplay1_DivisionalCancel_ControlStyleID></CustomDisplay1_DivisionalCancel_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay1_DivisionalCancel_PosPixels_X></CustomDisplay1_DivisionalCancel_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay1_DivisionalCancel_PosPixels_Y></CustomDisplay1_DivisionalCancel_PosPixels_Y>" & vbCrLf
            txt += "<CustomDisplay1_Keyboard_DisplayPageID>" & .displayPage & "</CustomDisplay1_Keyboard_DisplayPageID>" & vbCrLf
            txt += "<CustomDisplay1_Keyboard_KeyboardStyleID>" & .styleID & "</CustomDisplay1_Keyboard_KeyboardStyleID>" & vbCrLf
            txt += "<CustomDisplay1_Keyboard_PosPixels_X>" & .X & "</CustomDisplay1_Keyboard_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay1_Keyboard_PosPixels_Y>" & .Y & "</CustomDisplay1_Keyboard_PosPixels_Y>" & vbCrLf
            txt += "<CustomDisplay2_Divisionals_DisplayPageID></CustomDisplay2_Divisionals_DisplayPageID>" & vbCrLf
            txt += "<CustomDisplay2_Divisionals_NumberDisplayed>0</CustomDisplay2_Divisionals_NumberDisplayed>" & vbCrLf
            txt += "<CustomDisplay2_Divisionals_ControlStyleID></CustomDisplay2_Divisionals_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay2_Divisionals_FirstDisplayedPistonPos_X>0</CustomDisplay2_Divisionals_FirstDisplayedPistonPos_X>" & vbCrLf
            txt += "<CustomDisplay2_Divisionals_FirstDisplayedPistonPos_Y>0</CustomDisplay2_Divisionals_FirstDisplayedPistonPos_Y>" & vbCrLf
            txt += "<CustomDisplay2_Divisionals_LastDisplayedPistonPos_X>0</CustomDisplay2_Divisionals_LastDisplayedPistonPos_X>" & vbCrLf
            txt += "<CustomDisplay2_Divisionals_LastDisplayedPistonPos_Y>0</CustomDisplay2_Divisionals_LastDisplayedPistonPos_Y>" & vbCrLf
            txt += "<CustomDisplay2_DivisionalCancel_DisplayPageID></CustomDisplay2_DivisionalCancel_DisplayPageID>" & vbCrLf
            txt += "<CustomDisplay2_DivisionalCancel_Text></CustomDisplay2_DivisionalCancel_Text>" & vbCrLf
            txt += "<CustomDisplay2_DivisionalCancel_ControlStyleID></CustomDisplay2_DivisionalCancel_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay2_DivisionalCancel_PosPixels_X>0</CustomDisplay2_DivisionalCancel_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay2_DivisionalCancel_PosPixels_Y>0</CustomDisplay2_DivisionalCancel_PosPixels_Y>" & vbCrLf
            txt += "<CustomDisplay2_Keyboard_DisplayPageID></CustomDisplay2_Keyboard_DisplayPageID>" & vbCrLf
            txt += "<CustomDisplay2_Keyboard_KeyboardStyleID></CustomDisplay2_Keyboard_KeyboardStyleID>" & vbCrLf
            txt += "<CustomDisplay2_Keyboard_PosPixels_X>0</CustomDisplay2_Keyboard_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay2_Keyboard_PosPixels_Y>0</CustomDisplay2_Keyboard_PosPixels_Y>" & vbCrLf
            txt += "</division>" & vbCrLf
            txt += vbCrLf
        End With
        Main.RtbCODF.Text += txt
    End Sub
    Public Sub make_Manual()
        Main.RtbCODF.Text += "<ObjectList ObjectType = " & Chr(34) & "customdisplaykeyboardstyle" & Chr(34) & ">" & vbCrLf
        Main.RtbCODF.Text += "<customdisplaykeyboardstyle>" & vbCrLf
        Main.RtbCODF.Text += "<Name>manual 1</Name>" & vbCrLf
        Main.RtbCODF.Text += "<ImageFileInstallationPackageID>1512</ImageFileInstallationPackageID>" & vbCrLf
        Main.RtbCODF.Text += "<KeyboardStyleID>1</KeyboardStyleID>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_CF_KeyUp>setman001/CFup.bmp</KeyShapeImageFilename_CF_KeyUp>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_CF_KeyDown>setman001/CFdown.bmp</KeyShapeImageFilename_CF_KeyDown>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_CF_Mask>setman001/CFmask.bmp</KeyShapeImageFilename_CF_Mask>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_D_KeyUp>setman001/Dup.bmp</KeyShapeImageFilename_D_KeyUp>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_D_KeyDown>setman001/Ddown.bmp</KeyShapeImageFilename_D_KeyDown>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_D_Mask>setman001/Dmask.bmp</KeyShapeImageFilename_D_Mask>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_EB_KeyUp>setman001/EBup.bmp</KeyShapeImageFilename_EB_KeyUp>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_EB_KeyDown>setman001/EBdown.bmp</KeyShapeImageFilename_EB_KeyDown>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_EB_Mask>setman001/EBmask.bmp</KeyShapeImageFilename_EB_Mask>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_G_KeyUp>setman001/Gup.bmp</KeyShapeImageFilename_G_KeyUp>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_G_KeyDown>setman001/Gdown.bmp</KeyShapeImageFilename_G_KeyDown>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_G_Mask>setman001/Gmask.bmp</KeyShapeImageFilename_G_Mask>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_A_KeyUp>setman001/Aup.bmp</KeyShapeImageFilename_A_KeyUp>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_A_KeyDown>setman001/Adown.bmp</KeyShapeImageFilename_A_KeyDown>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_A_Mask>setman001/Amask.bmp</KeyShapeImageFilename_A_Mask>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_Sharp_KeyUp>setman001/sharpup.bmp</KeyShapeImageFilename_Sharp_KeyUp>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_Sharp_KeyDown>setman001/Sharpdown.bmp</KeyShapeImageFilename_Sharp_KeyDown>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_Sharp_Mask></KeyShapeImageFilename_Sharp_Mask>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_WholeNatural_KeyUp>setman001/Wholeup.bmp</KeyShapeImageFilename_WholeNatural_KeyUp>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_WholeNatural_KeyDown>setman001/wholedown.bmp</KeyShapeImageFilename_WholeNatural_KeyDown>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_WholeNatural_Mask></KeyShapeImageFilename_WholeNatural_Mask>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_FirstKeyDA_KeyUp>setman001/FirstKeyDAup.bmp</KeyShapeImageFilename_FirstKeyDA_KeyUp>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_FirstKeyDA_KeyDown>setman001/FirstKeyDAdown.bmp</KeyShapeImageFilename_FirstKeyDA_KeyDown>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_FirstKeyDA_Mask>setman001/FirstKeyDAmask.bmp</KeyShapeImageFilename_FirstKeyDA_Mask>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_FirstKeyG_KeyUp>setman001/FirstKeyGup.bmp</KeyShapeImageFilename_FirstKeyG_KeyUp>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_FirstKeyG_KeyDown>setman001/FirstKeyGdown.bmp</KeyShapeImageFilename_FirstKeyG_KeyDown>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_FirstKeyG_Mask>setman001/FirstKeyGmask.bmp</KeyShapeImageFilename_FirstKeyG_Mask>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_LastKeyDG_KeyUp>setman001/LastKeyDGup.bmp</KeyShapeImageFilename_LastKeyDG_KeyUp>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_LastKeyDG_KeyDown>setman001/LastKeyDGdown.bmp</KeyShapeImageFilename_LastKeyDG_KeyDown>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_LastKeyDG_Mask>setman001/EBmask.bmp</KeyShapeImageFilename_LastKeyDG_Mask>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_LastKeyA_KeyUp>setman001/LastKeyAup.bmp</KeyShapeImageFilename_LastKeyA_KeyUp>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_LastKeyA_KeyDown>setman001/LastKeyAdown.bmp</KeyShapeImageFilename_LastKeyA_KeyDown>" & vbCrLf
        Main.RtbCODF.Text += "<KeyShapeImageFilename_LastKeyA_Mask>setman001/LastKeyAmask.bmp</KeyShapeImageFilename_LastKeyA_Mask>" & vbCrLf
        Main.RtbCODF.Text += "<HorizSpacingPixels_LeftOfNaturalFromLeftOfNatural>12</HorizSpacingPixels_LeftOfNaturalFromLeftOfNatural>" & vbCrLf
        Main.RtbCODF.Text += "<HorizSpacingPixels_LeftOfCFSharpFromLeftOfCF>8</HorizSpacingPixels_LeftOfCFSharpFromLeftOfCF>" & vbCrLf
        Main.RtbCODF.Text += "<HorizSpacingPixels_LeftOfDASharpFromLeftOfDA>10</HorizSpacingPixels_LeftOfDASharpFromLeftOfDA>" & vbCrLf
        Main.RtbCODF.Text += "<HorizSpacingPixels_LeftOfGSharpFromLeftOfG>10</HorizSpacingPixels_LeftOfGSharpFromLeftOfG>" & vbCrLf
        Main.RtbCODF.Text += "<HorizSpacingPixels_LeftOfDGFromLeftOfCFSharp>4</HorizSpacingPixels_LeftOfDGFromLeftOfCFSharp>" & vbCrLf
        Main.RtbCODF.Text += "<HorizSpacingPixels_LeftOfEBFromLeftOfDASharp>2</HorizSpacingPixels_LeftOfEBFromLeftOfDASharp>" & vbCrLf
        Main.RtbCODF.Text += "<HorizSpacingPixels_LeftOfAFromLeftOfGSharp>2</HorizSpacingPixels_LeftOfAFromLeftOfGSharp>" & vbCrLf
        Main.RtbCODF.Text += "<ClickableAreaPosPixelsRelativeToTopLeftOfKey_Natural_LeftX>0</ClickableAreaPosPixelsRelativeToTopLeftOfKey_Natural_LeftX>" & vbCrLf
        Main.RtbCODF.Text += "<ClickableAreaPosPixelsRelativeToTopLeftOfKey_Natural_TopY>30</ClickableAreaPosPixelsRelativeToTopLeftOfKey_Natural_TopY>" & vbCrLf
        Main.RtbCODF.Text += "<ClickableAreaPosPixelsRelativeToTopLeftOfKey_Natural_RightX>12</ClickableAreaPosPixelsRelativeToTopLeftOfKey_Natural_RightX>" & vbCrLf
        Main.RtbCODF.Text += "<ClickableAreaPosPixelsRelativeToTopLeftOfKey_Natural_BottomY>51</ClickableAreaPosPixelsRelativeToTopLeftOfKey_Natural_BottomY>" & vbCrLf
        Main.RtbCODF.Text += "<ClickableAreaPosPixelsRelativeToTopLeftOfKey_Sharp_LeftX>0</ClickableAreaPosPixelsRelativeToTopLeftOfKey_Sharp_LeftX>" & vbCrLf
        Main.RtbCODF.Text += "<ClickableAreaPosPixelsRelativeToTopLeftOfKey_Sharp_TopY>0</ClickableAreaPosPixelsRelativeToTopLeftOfKey_Sharp_TopY>" & vbCrLf
        Main.RtbCODF.Text += "<ClickableAreaPosPixelsRelativeToTopLeftOfKey_Sharp_RightX>7</ClickableAreaPosPixelsRelativeToTopLeftOfKey_Sharp_RightX>" & vbCrLf
        Main.RtbCODF.Text += "<ClickableAreaPosPixelsRelativeToTopLeftOfKey_Sharp_BottomY>29</ClickableAreaPosPixelsRelativeToTopLeftOfKey_Sharp_BottomY>" & vbCrLf
        Main.RtbCODF.Text += "</customdisplaykeyboardstyle>" & vbCrLf
        Main.RtbCODF.Text += "</ObjectList>" & vbCrLf
        Main.RtbCODF.Text += "" & vbCrLf
    End Sub
    Public Sub make_coupler(ByVal cp As Integer)
        Dim txt As String = ""
        With coupler(cp)
            txt += "<coupler>" & vbCrLf
            txt += "<!--C" & .knobNumber & "-->" & vbCrLf
            txt += "<!--" & .KnobName & "-->" & vbCrLf
            txt += "<CouplerCode>" & .couplerCode & "</CouplerCode>" & vbCrLf
            txt += "<Name>Coupler: " & .name & "</Name>" & vbCrLf
            txt += "<CustomDisplay1_DisplayPageID>1</CustomDisplay1_DisplayPageID>" & vbCrLf
            txt += "<CustomDisplay1_Text>" & .name & "</CustomDisplay1_Text>" & vbCrLf
            txt += "<CustomDisplay1_ControlStyleID>" & .styleID1 & "</CustomDisplay1_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay1_PosPixels_X>" & .x1 & "</CustomDisplay1_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay1_PosPixels_Y>" & .y1 & "</CustomDisplay1_PosPixels_Y>" & vbCrLf
            txt += "<CustomDisplay1_AlternateLayout1_ControlStyleID></CustomDisplay1_AlternateLayout1_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay1_AlternateLayout1_PosPixels_X>0</CustomDisplay1_AlternateLayout1_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay1_AlternateLayout1_PosPixels_Y>0</CustomDisplay1_AlternateLayout1_PosPixels_Y>" & vbCrLf
            txt += "<CustomDisplay2_DisplayPageID>" & .jamb & "</CustomDisplay2_DisplayPageID>" & vbCrLf
            txt += "<CustomDisplay2_Text>" & .name & "</CustomDisplay2_Text>" & vbCrLf
            txt += "<CustomDisplay2_ControlStyleID>" & .styleID2 & "</CustomDisplay2_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay2_PosPixels_X>" & .x2 & "</CustomDisplay2_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay2_PosPixels_Y>" & .y2 & "</CustomDisplay2_PosPixels_Y>" & vbCrLf
            txt += "<CustomDisplay2_AlternateLayout1_ControlStyleID></CustomDisplay2_AlternateLayout1_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay2_AlternateLayout1_PosPixels_X>0</CustomDisplay2_AlternateLayout1_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay2_AlternateLayout1_PosPixels_Y>0</CustomDisplay2_AlternateLayout1_PosPixels_Y>" & vbCrLf
            txt += "<DisplayName>" & .name & "</DisplayName>" & vbCrLf
            txt += "<DefaultToEngaged>N</DefaultToEngaged>" & vbCrLf
            txt += "<SwitchIsLatching>Y</SwitchIsLatching>" & vbCrLf
            txt += "<StoreInDivisionalCombinations>Y</StoreInDivisionalCombinations>" & vbCrLf
            txt += "<StoreInGeneralCombinations>Y</StoreInGeneralCombinations>" & vbCrLf
            txt += "<AccessibleForMIDIInputAndOutput>Y</AccessibleForMIDIInputAndOutput>" & vbCrLf
            txt += "<PercussiveEngagingSoundEffect_RankID></PercussiveEngagingSoundEffect_RankID>" & vbCrLf
            txt += "<PercussiveEngagingSoundEffect_MIDINoteNumber>0</PercussiveEngagingSoundEffect_MIDINoteNumber>" & vbCrLf
            txt += "<PercussiveDisengagingSoundEffect_RankID></PercussiveDisengagingSoundEffect_RankID>" & vbCrLf
            txt += "<PercussiveDisengagingSoundEffect_MIDINoteNumber>0</PercussiveDisengagingSoundEffect_MIDINoteNumber>" & vbCrLf
            txt += "<SustainingSoundEffect_RankID></SustainingSoundEffect_RankID>" & vbCrLf
            txt += "<SustainingSoundEffect_MIDINoteNumber>0</SustainingSoundEffect_MIDINoteNumber>" & vbCrLf
            txt += "<OverrideSourceDivisionToSpecifiedDivisionCode></OverrideSourceDivisionToSpecifiedDivisionCode>" & vbCrLf
            txt += "<OverrideDestDivisionToSpecifiedDivisionCode></OverrideDestDivisionToSpecifiedDivisionCode>" & vbCrLf
            txt += "<CoupleDestinationKeyboardRatherThanKeyAction>N</CoupleDestinationKeyboardRatherThanKeyAction>" & vbCrLf
            txt += "<CouplerCodeFromWhichToCopyState></CouplerCodeFromWhichToCopyState>" & vbCrLf
            txt += "<CustomCoupler_MIDINoteNumIncrementFromSourceToDest>0</CustomCoupler_MIDINoteNumIncrementFromSourceToDest>" & vbCrLf
            txt += "<CustomCoupler_KeyActionTypeCode></CustomCoupler_KeyActionTypeCode>" & vbCrLf
            txt += "<CustomCoupler_KeyActionEffectCode></CustomCoupler_KeyActionEffectCode>" & vbCrLf
            txt += "<CustomCoupler_PipeMIDINoteNum036_PizzOrReitPeriodMs>0</CustomCoupler_PipeMIDINoteNum036_PizzOrReitPeriodMs>" & vbCrLf
            txt += "<CustomCoupler_PipeMIDINoteNum096_PizzOrReitPeriodMs>0</CustomCoupler_PipeMIDINoteNum096_PizzOrReitPeriodMs>" & vbCrLf
            txt += "</coupler>" & vbCrLf & vbCrLf
        End With
        Main.RtbCODF.Text += txt
    End Sub
    Public Sub make_enclosure(ByVal man As Integer)
        Dim txt As String = ""
        With enclosure(man)
            'txt += "<ObjectList ObjectType=" & Chr(34) & "enclosure" & Chr(34) & ">" & vbCrLf
            txt += "<enclosure>" & vbCrLf
            txt += "<EnclosureCode>" & .enclosureCode & "</EnclosureCode>" & vbCrLf
            txt += "<Name>" & .name & "</Name>" & vbCrLf
            txt += "<CustomDisplay1_DisplayPageID>1</CustomDisplay1_DisplayPageID>" & vbCrLf
            txt += "<CustomDisplay1_Text>" & .name & "</CustomDisplay1_Text>" & vbCrLf
            txt += "<CustomDisplay1_ControlStyleID>3</CustomDisplay1_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay1_PosPixels_X>" & .x1 & "</CustomDisplay1_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay1_PosPixels_Y>" & .y1 & "</CustomDisplay1_PosPixels_Y>" & vbCrLf
            txt += "<CustomDisplay1_AlternateLayout1_ControlStyleID></CustomDisplay1_AlternateLayout1_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay1_AlternateLayout1_PosPixels_X>0</CustomDisplay1_AlternateLayout1_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay1_AlternateLayout1_PosPixels_Y>0</CustomDisplay1_AlternateLayout1_PosPixels_Y>" & vbCrLf
            txt += "<CustomDisplay2_DisplayPageID></CustomDisplay2_DisplayPageID>" & vbCrLf
            txt += "<CustomDisplay2_Text></CustomDisplay2_Text>" & vbCrLf
            txt += "<CustomDisplay2_ControlStyleID></CustomDisplay2_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay2_PosPixels_X>0</CustomDisplay2_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay2_PosPixels_Y>0</CustomDisplay2_PosPixels_Y>" & vbCrLf
            txt += "<CustomDisplay2_AlternateLayout1_ControlStyleID></CustomDisplay2_AlternateLayout1_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay2_AlternateLayout1_PosPixels_X>0</CustomDisplay2_AlternateLayout1_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay2_AlternateLayout1_PosPixels_Y>0</CustomDisplay2_AlternateLayout1_PosPixels_Y>" & vbCrLf
            txt += "<EnableShutterInertiaModel>Y</EnableShutterInertiaModel>" & vbCrLf
            txt += "<WindModel_Enable>Y</WindModel_Enable>" & vbCrLf
            txt += "<WindModel_BoxPressureRisePctAtMaxLoadWhenClosed>2</WindModel_BoxPressureRisePctAtMaxLoadWhenClosed>" & vbCrLf
            txt += "</enclosure>" & vbCrLf
        End With
        Main.RtbCODF.Text += txt & vbCrLf & vbCrLf
    End Sub
    Public Sub make_CustomDisplayObjects()
        Dim txt As String = ""
        ' KNOBS RECTANGLE 90*30
        txt += "<customdisplaycontrolstyle>" & vbCrLf
        txt += "<ControlStyleID>" & buttonID & "</ControlStyleID>" & vbCrLf
        txt += "<Name>organ-stops</Name>" & vbCrLf
        txt += "<ImageFileInstallationPackageID>1512</ImageFileInstallationPackageID>" & vbCrLf
        txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_LeftX>0</ClickableAreaPosPixelsRelativeToTopLeftOfControl_LeftX>" & vbCrLf
        txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_TopY>0</ClickableAreaPosPixelsRelativeToTopLeftOfControl_TopY>" & vbCrLf
        txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_RightX>90</ClickableAreaPosPixelsRelativeToTopLeftOfControl_RightX>" & vbCrLf
        txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_Bottom>30</ClickableAreaPosPixelsRelativeToTopLeftOfControl_Bottom>" & vbCrLf
        txt += "<Image_TransparencyMaskFilename></Image_TransparencyMaskFilename>" & vbCrLf
        txt += "<Image_NumberOfStages>2</Image_NumberOfStages>" & vbCrLf
        txt += "<Image_Stage00_ImageFilename>images/" & buttonOff & "</Image_Stage00_ImageFilename>" & vbCrLf
        txt += "<Image_Stage01_ImageFilename>images/" & buttonOn & "</Image_Stage01_ImageFilename>" & vbCrLf
        txt += "<Image_Stage02_ImageFilename></Image_Stage02_ImageFilename>" & vbCrLf
        txt += "<Image_Stage03_ImageFilename></Image_Stage03_ImageFilename>" & vbCrLf
        txt += "<Image_Stage04_ImageFilename></Image_Stage04_ImageFilename>" & vbCrLf
        txt += "<Image_Stage05_ImageFilename></Image_Stage05_ImageFilename>" & vbCrLf
        txt += "<Image_Stage06_ImageFilename></Image_Stage06_ImageFilename>" & vbCrLf
        txt += "<Image_Stage07_ImageFilename></Image_Stage07_ImageFilename>" & vbCrLf
        txt += "<Image_Stage08_ImageFilename></Image_Stage08_ImageFilename>" & vbCrLf
        txt += "<Image_Stage09_ImageFilename></Image_Stage09_ImageFilename>" & vbCrLf
        txt += "<Image_Stage10_ImageFilename></Image_Stage10_ImageFilename>" & vbCrLf
        txt += "<Image_Stage11_ImageFilename></Image_Stage11_ImageFilename>" & vbCrLf
        txt += "<Image_Stage12_ImageFilename></Image_Stage12_ImageFilename>" & vbCrLf
        txt += "<Image_Stage13_ImageFilename></Image_Stage13_ImageFilename>" & vbCrLf
        txt += "<Image_Stage14_ImageFilename></Image_Stage14_ImageFilename>" & vbCrLf
        txt += "<Image_Stage15_ImageFilename></Image_Stage15_ImageFilename>" & vbCrLf
        txt += "<Text_HasText>Y</Text_HasText>" & vbCrLf
        txt += "<Text_Font_FaceName>" & knobfont.name & "</Text_Font_FaceName>" & vbCrLf
        txt += "<Text_Font_HeightPixels>" & Int(knobFont.Size) & "</Text_Font_HeightPixels>" & vbCrLf
        txt += "<Text_Font_WeightCode>2</Text_Font_WeightCode>" & vbCrLf
        txt += "<Text_Font_IsItalic>N</Text_Font_IsItalic>" & vbCrLf
        txt += "<Text_Font_IsUnderlined>N</Text_Font_IsUnderlined>" & vbCrLf
        txt += "<Text_RGBColour_Red>10</Text_RGBColour_Red>" & vbCrLf
        txt += "<Text_RGBColour_Green>10</Text_RGBColour_Green>" & vbCrLf
        txt += "<Text_RGBColour_Blue>40</Text_RGBColour_Blue>" & vbCrLf
        txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_LeftX>5</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_LeftX>" & vbCrLf
        txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_TopY>5</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_TopY>" & vbCrLf
        txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_RightX>85</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_RightX>" & vbCrLf
        txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_Bottom>25</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_Bottom>" & vbCrLf
        txt += "</customdisplaycontrolstyle>" & vbCrLf
        txt += "" & vbCrLf
        'ROUND KNOB 72 PIXELS
        If Main.RadioButton1.Checked = True Then
            txt += "<customdisplaycontrolstyle>" & vbCrLf
            txt += "<ControlStyleID>" & jambButtonID & "</ControlStyleID>" & vbCrLf
            txt += "<Name>jamb-stops</Name>" & vbCrLf
            txt += "<ImageFileInstallationPackageID>1512</ImageFileInstallationPackageID>" & vbCrLf
            txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_LeftX>0</ClickableAreaPosPixelsRelativeToTopLeftOfControl_LeftX>" & vbCrLf
            txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_TopY>0</ClickableAreaPosPixelsRelativeToTopLeftOfControl_TopY>" & vbCrLf
            txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_RightX>72</ClickableAreaPosPixelsRelativeToTopLeftOfControl_RightX>" & vbCrLf
            txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_Bottom>72</ClickableAreaPosPixelsRelativeToTopLeftOfControl_Bottom>" & vbCrLf
            txt += "<Image_TransparencyMaskFilename>images/" & jambButtonMask & "</Image_TransparencyMaskFilename>" & vbCrLf
            txt += "<Image_NumberOfStages>2</Image_NumberOfStages>" & vbCrLf
            txt += "<Image_Stage00_ImageFilename>images/" & jambButtonOff & "</Image_Stage00_ImageFilename>" & vbCrLf
            txt += "<Image_Stage01_ImageFilename>images/" & jambButtonOn & "</Image_Stage01_ImageFilename>" & vbCrLf
            txt += "<Image_Stage02_ImageFilename></Image_Stage02_ImageFilename>" & vbCrLf
            txt += "<Image_Stage03_ImageFilename></Image_Stage03_ImageFilename>" & vbCrLf
            txt += "<Image_Stage04_ImageFilename></Image_Stage04_ImageFilename>" & vbCrLf
            txt += "<Image_Stage05_ImageFilename></Image_Stage05_ImageFilename>" & vbCrLf
            txt += "<Image_Stage06_ImageFilename></Image_Stage06_ImageFilename>" & vbCrLf
            txt += "<Image_Stage07_ImageFilename></Image_Stage07_ImageFilename>" & vbCrLf
            txt += "<Image_Stage08_ImageFilename></Image_Stage08_ImageFilename>" & vbCrLf
            txt += "<Image_Stage09_ImageFilename></Image_Stage09_ImageFilename>" & vbCrLf
            txt += "<Image_Stage10_ImageFilename></Image_Stage10_ImageFilename>" & vbCrLf
            txt += "<Image_Stage11_ImageFilename></Image_Stage11_ImageFilename>" & vbCrLf
            txt += "<Image_Stage12_ImageFilename></Image_Stage12_ImageFilename>" & vbCrLf
            txt += "<Image_Stage13_ImageFilename></Image_Stage13_ImageFilename>" & vbCrLf
            txt += "<Image_Stage14_ImageFilename></Image_Stage14_ImageFilename>" & vbCrLf
            txt += "<Image_Stage15_ImageFilename></Image_Stage15_ImageFilename>" & vbCrLf
            txt += "<Text_HasText>Y</Text_HasText>" & vbCrLf
            txt += "<Text_Font_FaceName>" & jambFont.Name & "</Text_Font_FaceName>" & vbCrLf
            txt += "<Text_Font_HeightPixels>" & Int(jambFont.Size) & "</Text_Font_HeightPixels>" & vbCrLf
            txt += "<Text_Font_WeightCode>2</Text_Font_WeightCode>" & vbCrLf
            txt += "<Text_Font_IsItalic>N</Text_Font_IsItalic>" & vbCrLf
            txt += "<Text_Font_IsUnderlined>N</Text_Font_IsUnderlined>" & vbCrLf
            txt += "<Text_RGBColour_Red>10</Text_RGBColour_Red>" & vbCrLf
            txt += "<Text_RGBColour_Green>10</Text_RGBColour_Green>" & vbCrLf
            txt += "<Text_RGBColour_Blue>40</Text_RGBColour_Blue>" & vbCrLf
            txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_LeftX>5</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_LeftX>" & vbCrLf
            txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_TopY>25</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_TopY>" & vbCrLf
            txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_RightX>65</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_RightX>" & vbCrLf
            txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_Bottom>55</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_Bottom>" & vbCrLf
            txt += "</customdisplaycontrolstyle>" & vbCrLf & vbCrLf
        End If
        '
        'ROUND KNOB 84 PIXELS
        If Main.RadioButton2.Checked = True Or Main.RadioButton3.Checked = True Then
            txt += "<customdisplaycontrolstyle>" & vbCrLf
            txt += "<ControlStyleID>" & jambButtonID & "</ControlStyleID>" & vbCrLf
            txt += "<Name>jamb-stops 84x84</Name>" & vbCrLf
            txt += "<ImageFileInstallationPackageID>1512</ImageFileInstallationPackageID>" & vbCrLf
            txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_LeftX>0</ClickableAreaPosPixelsRelativeToTopLeftOfControl_LeftX>" & vbCrLf
            txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_TopY>0</ClickableAreaPosPixelsRelativeToTopLeftOfControl_TopY>" & vbCrLf
            txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_RightX>84</ClickableAreaPosPixelsRelativeToTopLeftOfControl_RightX>" & vbCrLf
            txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_Bottom>84</ClickableAreaPosPixelsRelativeToTopLeftOfControl_Bottom>" & vbCrLf
            txt += "<Image_TransparencyMaskFilename>images/" & jambButtonMask & "</Image_TransparencyMaskFilename>" & vbCrLf
            txt += "<Image_NumberOfStages>2</Image_NumberOfStages>" & vbCrLf
            txt += "<Image_Stage00_ImageFilename>images/" & jambButtonOff & "</Image_Stage00_ImageFilename>" & vbCrLf
            txt += "<Image_Stage01_ImageFilename>images/" & jambButtonOn & "</Image_Stage01_ImageFilename>" & vbCrLf
            txt += "<Image_Stage02_ImageFilename></Image_Stage02_ImageFilename>" & vbCrLf
            txt += "<Image_Stage03_ImageFilename></Image_Stage03_ImageFilename>" & vbCrLf
            txt += "<Image_Stage04_ImageFilename></Image_Stage04_ImageFilename>" & vbCrLf
            txt += "<Image_Stage05_ImageFilename></Image_Stage05_ImageFilename>" & vbCrLf
            txt += "<Image_Stage06_ImageFilename></Image_Stage06_ImageFilename>" & vbCrLf
            txt += "<Image_Stage07_ImageFilename></Image_Stage07_ImageFilename>" & vbCrLf
            txt += "<Image_Stage08_ImageFilename></Image_Stage08_ImageFilename>" & vbCrLf
            txt += "<Image_Stage09_ImageFilename></Image_Stage09_ImageFilename>" & vbCrLf
            txt += "<Image_Stage10_ImageFilename></Image_Stage10_ImageFilename>" & vbCrLf
            txt += "<Image_Stage11_ImageFilename></Image_Stage11_ImageFilename>" & vbCrLf
            txt += "<Image_Stage12_ImageFilename></Image_Stage12_ImageFilename>" & vbCrLf
            txt += "<Image_Stage13_ImageFilename></Image_Stage13_ImageFilename>" & vbCrLf
            txt += "<Image_Stage14_ImageFilename></Image_Stage14_ImageFilename>" & vbCrLf
            txt += "<Image_Stage15_ImageFilename></Image_Stage15_ImageFilename>" & vbCrLf
            txt += "<Text_HasText>Y</Text_HasText>" & vbCrLf
            txt += "<Text_Font_FaceName>" & jambFont.Name & "</Text_Font_FaceName>" & vbCrLf
            txt += "<Text_Font_HeightPixels>" & Int(jambFont.Size) & "</Text_Font_HeightPixels>" & vbCrLf
            txt += "<Text_Font_WeightCode>2</Text_Font_WeightCode>" & vbCrLf
            txt += "<Text_Font_IsItalic>N</Text_Font_IsItalic>" & vbCrLf
            txt += "<Text_Font_IsUnderlined>N</Text_Font_IsUnderlined>" & vbCrLf
            txt += "<Text_RGBColour_Red>10</Text_RGBColour_Red>" & vbCrLf
            txt += "<Text_RGBColour_Green>10</Text_RGBColour_Green>" & vbCrLf
            txt += "<Text_RGBColour_Blue>40</Text_RGBColour_Blue>" & vbCrLf
            txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_LeftX>5</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_LeftX>" & vbCrLf
            txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_TopY>25</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_TopY>" & vbCrLf
            txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_RightX>78</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_RightX>" & vbCrLf
            txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_Bottom>65</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_Bottom>" & vbCrLf
            txt += "</customdisplaycontrolstyle>" & vbCrLf & vbCrLf
        End If

        'TAB WHITE 80x100 PIXEL
        If Main.RadioButton4.Checked = True Then
            txt += "<customdisplaycontrolstyle>" & vbCrLf
            txt += "<ControlStyleID>" & jambButtonID & "</ControlStyleID>" & vbCrLf
            txt += "<Name>Tab-white80x100stops</Name>" & vbCrLf
            txt += "<ImageFileInstallationPackageID>1512</ImageFileInstallationPackageID>" & vbCrLf
            txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_LeftX>0</ClickableAreaPosPixelsRelativeToTopLeftOfControl_LeftX>" & vbCrLf
            txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_TopY>0</ClickableAreaPosPixelsRelativeToTopLeftOfControl_TopY>" & vbCrLf
            txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_RightX>80</ClickableAreaPosPixelsRelativeToTopLeftOfControl_RightX>" & vbCrLf
            txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_Bottom>100</ClickableAreaPosPixelsRelativeToTopLeftOfControl_Bottom>" & vbCrLf
            txt += "<Image_TransparencyMaskFilename>images/" & jambButtonMask & "</Image_TransparencyMaskFilename>" & vbCrLf
            txt += "<Image_NumberOfStages>2</Image_NumberOfStages>" & vbCrLf
            txt += "<Image_Stage00_ImageFilename>images/" & jambButtonOff & "</Image_Stage00_ImageFilename>" & vbCrLf
            txt += "<Image_Stage01_ImageFilename>images/" & jambButtonOn & "</Image_Stage01_ImageFilename>" & vbCrLf
            txt += "<Image_Stage02_ImageFilename></Image_Stage02_ImageFilename>" & vbCrLf
            txt += "<Image_Stage03_ImageFilename></Image_Stage03_ImageFilename>" & vbCrLf
            txt += "<Image_Stage04_ImageFilename></Image_Stage04_ImageFilename>" & vbCrLf
            txt += "<Image_Stage05_ImageFilename></Image_Stage05_ImageFilename>" & vbCrLf
            txt += "<Image_Stage06_ImageFilename></Image_Stage06_ImageFilename>" & vbCrLf
            txt += "<Image_Stage07_ImageFilename></Image_Stage07_ImageFilename>" & vbCrLf
            txt += "<Image_Stage08_ImageFilename></Image_Stage08_ImageFilename>" & vbCrLf
            txt += "<Image_Stage09_ImageFilename></Image_Stage09_ImageFilename>" & vbCrLf
            txt += "<Image_Stage10_ImageFilename></Image_Stage10_ImageFilename>" & vbCrLf
            txt += "<Image_Stage11_ImageFilename></Image_Stage11_ImageFilename>" & vbCrLf
            txt += "<Image_Stage12_ImageFilename></Image_Stage12_ImageFilename>" & vbCrLf
            txt += "<Image_Stage13_ImageFilename></Image_Stage13_ImageFilename>" & vbCrLf
            txt += "<Image_Stage14_ImageFilename></Image_Stage14_ImageFilename>" & vbCrLf
            txt += "<Image_Stage15_ImageFilename></Image_Stage15_ImageFilename>" & vbCrLf
            txt += "<Text_HasText>Y</Text_HasText>" & vbCrLf
            txt += "<Text_Font_FaceName>" & jambFont.Name & "</Text_Font_FaceName>" & vbCrLf
            txt += "<Text_Font_HeightPixels>" & Int(jambFont.Size) & "</Text_Font_HeightPixels>" & vbCrLf
            txt += "<Text_Font_WeightCode>2</Text_Font_WeightCode>" & vbCrLf
            txt += "<Text_Font_IsItalic>N</Text_Font_IsItalic>" & vbCrLf
            txt += "<Text_Font_IsUnderlined>N</Text_Font_IsUnderlined>" & vbCrLf
            txt += "<Text_RGBColour_Red>10</Text_RGBColour_Red>" & vbCrLf
            txt += "<Text_RGBColour_Green>10</Text_RGBColour_Green>" & vbCrLf
            txt += "<Text_RGBColour_Blue>40</Text_RGBColour_Blue>" & vbCrLf
            txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_LeftX>5</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_LeftX>" & vbCrLf
            txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_TopY>25</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_TopY>" & vbCrLf
            txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_RightX>75</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_RightX>" & vbCrLf
            txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_Bottom>70</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_Bottom>" & vbCrLf
            txt += "</customdisplaycontrolstyle>" & vbCrLf & vbCrLf
        End If

        ' LABELS

        txt += "<customdisplaycontrolstyle>" & vbCrLf
        txt += "<ControlStyleID>12</ControlStyleID>" & vbCrLf
        txt += "<Name>Label 36x18</Name>" & vbCrLf
        txt += "<ImageFileInstallationPackageID>1512</ImageFileInstallationPackageID>" & vbCrLf
        txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_LeftX>0</ClickableAreaPosPixelsRelativeToTopLeftOfControl_LeftX>" & vbCrLf
        txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_TopY>0</ClickableAreaPosPixelsRelativeToTopLeftOfControl_TopY>" & vbCrLf
        txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_RightX>36</ClickableAreaPosPixelsRelativeToTopLeftOfControl_RightX>" & vbCrLf
        txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_Bottom>18</ClickableAreaPosPixelsRelativeToTopLeftOfControl_Bottom>" & vbCrLf
        txt += "<Image_TransparencyMaskFilename></Image_TransparencyMaskFilename>" & vbCrLf
        txt += "<Image_NumberOfStages>1</Image_NumberOfStages>" & vbCrLf
        txt += "<Image_Stage00_ImageFilename>images/Label-001-36x18.png</Image_Stage00_ImageFilename>" & vbCrLf
        txt += "<Image_Stage01_ImageFilename></Image_Stage01_ImageFilename>" & vbCrLf
        txt += "<Image_Stage02_ImageFilename></Image_Stage02_ImageFilename>" & vbCrLf
        txt += "<Image_Stage03_ImageFilename></Image_Stage03_ImageFilename>" & vbCrLf
        txt += "<Image_Stage04_ImageFilename></Image_Stage04_ImageFilename>" & vbCrLf
        txt += "<Image_Stage05_ImageFilename></Image_Stage05_ImageFilename>" & vbCrLf
        txt += "<Image_Stage06_ImageFilename></Image_Stage06_ImageFilename>" & vbCrLf
        txt += "<Image_Stage07_ImageFilename></Image_Stage07_ImageFilename>" & vbCrLf
        txt += "<Image_Stage08_ImageFilename></Image_Stage08_ImageFilename>" & vbCrLf
        txt += "<Image_Stage09_ImageFilename></Image_Stage09_ImageFilename>" & vbCrLf
        txt += "<Image_Stage10_ImageFilename></Image_Stage10_ImageFilename>" & vbCrLf
        txt += "<Image_Stage11_ImageFilename></Image_Stage11_ImageFilename>" & vbCrLf
        txt += "<Image_Stage12_ImageFilename></Image_Stage12_ImageFilename>" & vbCrLf
        txt += "<Image_Stage13_ImageFilename></Image_Stage13_ImageFilename>" & vbCrLf
        txt += "<Image_Stage14_ImageFilename></Image_Stage14_ImageFilename>" & vbCrLf
        txt += "<Image_Stage15_ImageFilename></Image_Stage15_ImageFilename>" & vbCrLf
        txt += "<Text_HasText>Y</Text_HasText>" & vbCrLf
        txt += "<Text_Font_FaceName>Verdana</Text_Font_FaceName>" & vbCrLf
        txt += "<Text_Font_HeightPixels>10</Text_Font_HeightPixels>" & vbCrLf
        txt += "<Text_Font_WeightCode>2</Text_Font_WeightCode>" & vbCrLf
        txt += "<Text_Font_IsItalic>N</Text_Font_IsItalic>" & vbCrLf
        txt += "<Text_Font_IsUnderlined>N</Text_Font_IsUnderlined>" & vbCrLf
        txt += "<Text_RGBColour_Red>255</Text_RGBColour_Red>" & vbCrLf
        txt += "<Text_RGBColour_Green>255</Text_RGBColour_Green>" & vbCrLf
        txt += "<Text_RGBColour_Blue>255</Text_RGBColour_Blue>" & vbCrLf
        txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_LeftX>2</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_LeftX>" & vbCrLf
        txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_TopY>2</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_TopY>" & vbCrLf
        txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_RightX>34</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_RightX>" & vbCrLf
        txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_Bottom>16</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_Bottom>" & vbCrLf
        txt += "</customdisplaycontrolstyle>" & vbCrLf
        '
        txt += "<customdisplaycontrolstyle>" & vbCrLf
        txt += "<ControlStyleID>13</ControlStyleID>" & vbCrLf
        txt += "<Name>Label 58x20</Name>" & vbCrLf
        txt += "<ImageFileInstallationPackageID>1512</ImageFileInstallationPackageID>" & vbCrLf
        txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_LeftX>0</ClickableAreaPosPixelsRelativeToTopLeftOfControl_LeftX>" & vbCrLf
        txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_TopY>0</ClickableAreaPosPixelsRelativeToTopLeftOfControl_TopY>" & vbCrLf
        txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_RightX>36</ClickableAreaPosPixelsRelativeToTopLeftOfControl_RightX>" & vbCrLf
        txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_Bottom>18</ClickableAreaPosPixelsRelativeToTopLeftOfControl_Bottom>" & vbCrLf
        txt += "<Image_TransparencyMaskFilename></Image_TransparencyMaskFilename>" & vbCrLf
        txt += "<Image_NumberOfStages>1</Image_NumberOfStages>" & vbCrLf
        txt += "<Image_Stage00_ImageFilename>images/Label-001-58x20.png</Image_Stage00_ImageFilename>" & vbCrLf
        txt += "<Image_Stage01_ImageFilename></Image_Stage01_ImageFilename>" & vbCrLf
        txt += "<Image_Stage02_ImageFilename></Image_Stage02_ImageFilename>" & vbCrLf
        txt += "<Image_Stage03_ImageFilename></Image_Stage03_ImageFilename>" & vbCrLf
        txt += "<Image_Stage04_ImageFilename></Image_Stage04_ImageFilename>" & vbCrLf
        txt += "<Image_Stage05_ImageFilename></Image_Stage05_ImageFilename>" & vbCrLf
        txt += "<Image_Stage06_ImageFilename></Image_Stage06_ImageFilename>" & vbCrLf
        txt += "<Image_Stage07_ImageFilename></Image_Stage07_ImageFilename>" & vbCrLf
        txt += "<Image_Stage08_ImageFilename></Image_Stage08_ImageFilename>" & vbCrLf
        txt += "<Image_Stage09_ImageFilename></Image_Stage09_ImageFilename>" & vbCrLf
        txt += "<Image_Stage10_ImageFilename></Image_Stage10_ImageFilename>" & vbCrLf
        txt += "<Image_Stage11_ImageFilename></Image_Stage11_ImageFilename>" & vbCrLf
        txt += "<Image_Stage12_ImageFilename></Image_Stage12_ImageFilename>" & vbCrLf
        txt += "<Image_Stage13_ImageFilename></Image_Stage13_ImageFilename>" & vbCrLf
        txt += "<Image_Stage14_ImageFilename></Image_Stage14_ImageFilename>" & vbCrLf
        txt += "<Image_Stage15_ImageFilename></Image_Stage15_ImageFilename>" & vbCrLf
        txt += "<Text_HasText>Y</Text_HasText>" & vbCrLf
        txt += "<Text_Font_FaceName>Verdana</Text_Font_FaceName>" & vbCrLf
        txt += "<Text_Font_HeightPixels>12</Text_Font_HeightPixels>" & vbCrLf
        txt += "<Text_Font_WeightCode>2</Text_Font_WeightCode>" & vbCrLf
        txt += "<Text_Font_IsItalic>N</Text_Font_IsItalic>" & vbCrLf
        txt += "<Text_Font_IsUnderlined>N</Text_Font_IsUnderlined>" & vbCrLf
        txt += "<Text_RGBColour_Red>255</Text_RGBColour_Red>" & vbCrLf
        txt += "<Text_RGBColour_Green>255</Text_RGBColour_Green>" & vbCrLf
        txt += "<Text_RGBColour_Blue>255</Text_RGBColour_Blue>" & vbCrLf
        txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_LeftX>2</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_LeftX>" & vbCrLf
        txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_TopY>2</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_TopY>" & vbCrLf
        txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_RightX>56</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_RightX>" & vbCrLf
        txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_Bottom>18</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_Bottom>" & vbCrLf
        txt += "</customdisplaycontrolstyle>" & vbCrLf
        '
        ' BANDE HORIZONTALES
        txt += "<customdisplaycontrolstyle>" & vbCrLf
        txt += "<ControlStyleID>20</ControlStyleID>" & vbCrLf
        txt += "<Name>Label BandeH</Name>" & vbCrLf
        txt += "<ImageFileInstallationPackageID>1512</ImageFileInstallationPackageID>" & vbCrLf
        txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_LeftX>0</ClickableAreaPosPixelsRelativeToTopLeftOfControl_LeftX>" & vbCrLf
        txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_TopY>0</ClickableAreaPosPixelsRelativeToTopLeftOfControl_TopY>" & vbCrLf
        txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_RightX>0</ClickableAreaPosPixelsRelativeToTopLeftOfControl_RightX>" & vbCrLf
        txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_Bottom>0</ClickableAreaPosPixelsRelativeToTopLeftOfControl_Bottom>" & vbCrLf
        txt += "<Image_TransparencyMaskFilename></Image_TransparencyMaskFilename>" & vbCrLf
        txt += "<Image_NumberOfStages>1</Image_NumberOfStages>" & vbCrLf
        txt += "<Image_Stage00_ImageFilename>images/BandeH.png</Image_Stage00_ImageFilename>" & vbCrLf
        txt += "<Image_Stage01_ImageFilename></Image_Stage01_ImageFilename>" & vbCrLf
        txt += "<Image_Stage02_ImageFilename></Image_Stage02_ImageFilename>" & vbCrLf
        txt += "<Image_Stage03_ImageFilename></Image_Stage03_ImageFilename>" & vbCrLf
        txt += "<Image_Stage04_ImageFilename></Image_Stage04_ImageFilename>" & vbCrLf
        txt += "<Image_Stage05_ImageFilename></Image_Stage05_ImageFilename>" & vbCrLf
        txt += "<Image_Stage06_ImageFilename></Image_Stage06_ImageFilename>" & vbCrLf
        txt += "<Image_Stage07_ImageFilename></Image_Stage07_ImageFilename>" & vbCrLf
        txt += "<Image_Stage08_ImageFilename></Image_Stage08_ImageFilename>" & vbCrLf
        txt += "<Image_Stage09_ImageFilename></Image_Stage09_ImageFilename>" & vbCrLf
        txt += "<Image_Stage10_ImageFilename></Image_Stage10_ImageFilename>" & vbCrLf
        txt += "<Image_Stage11_ImageFilename></Image_Stage11_ImageFilename>" & vbCrLf
        txt += "<Image_Stage12_ImageFilename></Image_Stage12_ImageFilename>" & vbCrLf
        txt += "<Image_Stage13_ImageFilename></Image_Stage13_ImageFilename>" & vbCrLf
        txt += "<Image_Stage14_ImageFilename></Image_Stage14_ImageFilename>" & vbCrLf
        txt += "<Image_Stage15_ImageFilename></Image_Stage15_ImageFilename>" & vbCrLf
        txt += "<Text_HasText>n</Text_HasText>" & vbCrLf
        txt += "<Text_Font_FaceName></Text_Font_FaceName>" & vbCrLf
        txt += "<Text_Font_HeightPixels></Text_Font_HeightPixels>" & vbCrLf
        txt += "<Text_Font_WeightCode></Text_Font_WeightCode>" & vbCrLf
        txt += "<Text_Font_IsItalic></Text_Font_IsItalic>" & vbCrLf
        txt += "<Text_Font_IsUnderlined></Text_Font_IsUnderlined>" & vbCrLf
        txt += "<Text_RGBColour_Red></Text_RGBColour_Red>" & vbCrLf
        txt += "<Text_RGBColour_Green></Text_RGBColour_Green>" & vbCrLf
        txt += "<Text_RGBColour_Blue></Text_RGBColour_Blue>" & vbCrLf
        txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_LeftX></Text_BoundingBoxPosPixelsRelToTopLeftOfImage_LeftX>" & vbCrLf
        txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_TopY></Text_BoundingBoxPosPixelsRelToTopLeftOfImage_TopY>" & vbCrLf
        txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_RightX></Text_BoundingBoxPosPixelsRelToTopLeftOfImage_RightX>" & vbCrLf
        txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_Bottom></Text_BoundingBoxPosPixelsRelToTopLeftOfImage_Bottom>" & vbCrLf
        txt += "</customdisplaycontrolstyle>" & vbCrLf


        ' FIN DE SECTION
        Main.RtbCODF.Text += txt
    End Sub
    Public Sub make_labels(ByVal lb As Integer)

        Dim txt As String = ""
        With label(lb)
            txt += "<customdisplaylabel>" & vbCrLf
            txt += "<Name>" & .name & "</Name>" & vbCrLf
            txt += "<CustomDisplay1_DisplayPageID>" & .page & "</CustomDisplay1_DisplayPageID>" & vbCrLf
            txt += "<CustomDisplay1_Text>" & .text & "</CustomDisplay1_Text>" & vbCrLf
            txt += "<CustomDisplay1_ControlStyleID>13</CustomDisplay1_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay1_PosPixels_X>" & .posX & "</CustomDisplay1_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay1_PosPixels_Y>" & .posY & "</CustomDisplay1_PosPixels_Y>" & vbCrLf
            txt += "<CustomDisplay1_AlternateLayout1_ControlStyleID></CustomDisplay1_AlternateLayout1_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay1_AlternateLayout1_PosPixels_X>0</CustomDisplay1_AlternateLayout1_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay1_AlternateLayout1_PosPixels_Y>0</CustomDisplay1_AlternateLayout1_PosPixels_Y>" & vbCrLf
            txt += "<CustomDisplay2_DisplayPageID></CustomDisplay2_DisplayPageID>" & vbCrLf
            txt += "<CustomDisplay2_Text></CustomDisplay2_Text>" & vbCrLf
            txt += "<CustomDisplay2_ControlStyleID></CustomDisplay2_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay2_PosPixels_X>0</CustomDisplay2_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay2_PosPixels_Y>0</CustomDisplay2_PosPixels_Y>" & vbCrLf
            txt += "<CustomDisplay2_AlternateLayout1_ControlStyleID></CustomDisplay2_AlternateLayout1_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay2_AlternateLayout1_PosPixels_X>0</CustomDisplay2_AlternateLayout1_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay2_AlternateLayout1_PosPixels_Y>0</CustomDisplay2_AlternateLayout1_PosPixels_Y>" & vbCrLf
            txt += "</customdisplaylabel>" & vbCrLf
        End With
        ' FIN DE SECTION
        Main.RtbCODF.Text += txt
        
    End Sub
    Public Sub make_bandeH()
        Dim txt As String = ""

        Dim bh As Single = 0
        bandeH(0).posX = 1
        bandeH(0).posY = 447
        bandeH(0).jamb = 2
        bandeH(1).posX = 1
        bandeH(1).posY = 233
        If nbrOfManuals = 3 Then bandeH(1).posY = 126
        If nbrOfManuals < 3 Then bandeH(1).posY = 447 ' sera confondu avec bandeH 0
        bandeH(1).jamb = 2
        bandeH(2).posX = 1
        bandeH(2).posY = 447
        If nbrOfManuals = 2 Then bandeH(2).posY = 340
        bandeH(2).jamb = 3
        bandeH(3).posX = 1
        bandeH(3).posY = 233
        bandeH(3).jamb = 3

        For bh = 0 To 3
            With bandeH(bh)
                txt += "<customdisplaylabel>" & vbCrLf
                txt += "<Name>" & "bande " & bh.ToString & "</Name>" & vbCrLf
                txt += "<CustomDisplay1_DisplayPageID>" & "</CustomDisplay1_DisplayPageID>" & vbCrLf
                txt += "<CustomDisplay1_Text>" & "</CustomDisplay1_Text>" & vbCrLf
                txt += "<CustomDisplay1_ControlStyleID></CustomDisplay1_ControlStyleID>" & vbCrLf
                txt += "<CustomDisplay1_PosPixels_X></CustomDisplay1_PosPixels_X>" & vbCrLf
                txt += "<CustomDisplay1_PosPixels_Y>" & "</CustomDisplay1_PosPixels_Y>" & vbCrLf
                txt += "<CustomDisplay1_AlternateLayout1_ControlStyleID></CustomDisplay1_AlternateLayout1_ControlStyleID>" & vbCrLf
                txt += "<CustomDisplay1_AlternateLayout1_PosPixels_X>0</CustomDisplay1_AlternateLayout1_PosPixels_X>" & vbCrLf
                txt += "<CustomDisplay1_AlternateLayout1_PosPixels_Y>0</CustomDisplay1_AlternateLayout1_PosPixels_Y>" & vbCrLf
                txt += "<CustomDisplay2_DisplayPageID>" & .jamb & "</CustomDisplay2_DisplayPageID>" & vbCrLf
                txt += "<CustomDisplay2_Text></CustomDisplay2_Text>" & vbCrLf
                txt += "<CustomDisplay2_ControlStyleID>20</CustomDisplay2_ControlStyleID>" & vbCrLf
                txt += "<CustomDisplay2_PosPixels_X>" & .posX & "</CustomDisplay2_PosPixels_X>" & vbCrLf
                txt += "<CustomDisplay2_PosPixels_Y>" & .posY & "</CustomDisplay2_PosPixels_Y>" & vbCrLf
                txt += "<CustomDisplay2_AlternateLayout1_ControlStyleID></CustomDisplay2_AlternateLayout1_ControlStyleID>" & vbCrLf
                txt += "<CustomDisplay2_AlternateLayout1_PosPixels_X>0</CustomDisplay2_AlternateLayout1_PosPixels_X>" & vbCrLf
                txt += "<CustomDisplay2_AlternateLayout1_PosPixels_Y>0</CustomDisplay2_AlternateLayout1_PosPixels_Y>" & vbCrLf
                txt += "</customdisplaylabel>" & vbCrLf
            End With
        Next
        ' FIN DE SECTION
        Main.RtbCODF.Text += txt
    End Sub
    Public Sub make_customExpPedal()
        Dim txt As String = ""
        txt += "<customdisplaycontrolstyle>" & vbCrLf
        txt += "<ControlStyleID>3</ControlStyleID>" & vbCrLf
        txt += "<Name>ExpressionPedal</Name>" & vbCrLf
        txt += "<ImageFileInstallationPackageID>1512</ImageFileInstallationPackageID>" & vbCrLf
        txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_LeftX>0</ClickableAreaPosPixelsRelativeToTopLeftOfControl_LeftX>" & vbCrLf
        txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_TopY>0</ClickableAreaPosPixelsRelativeToTopLeftOfControl_TopY>" & vbCrLf
        txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_RightX>46</ClickableAreaPosPixelsRelativeToTopLeftOfControl_RightX>" & vbCrLf
        txt += "<ClickableAreaPosPixelsRelativeToTopLeftOfControl_Bottom>87</ClickableAreaPosPixelsRelativeToTopLeftOfControl_Bottom>" & vbCrLf
        txt += "<Image_TransparencyMaskFilename></Image_TransparencyMaskFilename>" & vbCrLf
        txt += "<Image_NumberOfStages>16</Image_NumberOfStages>" & vbCrLf
        txt += "<Image_Stage00_ImageFilename>images/expressionPedal/expres01.bmp</Image_Stage00_ImageFilename>" & vbCrLf
        txt += "<Image_Stage01_ImageFilename>images/expressionPedal/expres02.bmp</Image_Stage01_ImageFilename>" & vbCrLf
        txt += "<Image_Stage02_ImageFilename>images/expressionPedal/expres03.bmp</Image_Stage02_ImageFilename>" & vbCrLf
        txt += "<Image_Stage03_ImageFilename>images/expressionPedal/expres04.bmp</Image_Stage03_ImageFilename>" & vbCrLf
        txt += "<Image_Stage04_ImageFilename>images/expressionPedal/expres05.bmp</Image_Stage04_ImageFilename>" & vbCrLf
        txt += "<Image_Stage05_ImageFilename>images/expressionPedal/expres06.bmp</Image_Stage05_ImageFilename>" & vbCrLf
        txt += "<Image_Stage06_ImageFilename>images/expressionPedal/expres07.bmp</Image_Stage06_ImageFilename>" & vbCrLf
        txt += "<Image_Stage07_ImageFilename>images/expressionPedal/expres08.bmp</Image_Stage07_ImageFilename>" & vbCrLf
        txt += "<Image_Stage08_ImageFilename>images/expressionPedal/expres09.bmp</Image_Stage08_ImageFilename>" & vbCrLf
        txt += "<Image_Stage09_ImageFilename>images/expressionPedal/expres10.bmp</Image_Stage09_ImageFilename>" & vbCrLf
        txt += "<Image_Stage10_ImageFilename>images/expressionPedal/expres11.bmp</Image_Stage10_ImageFilename>" & vbCrLf
        txt += "<Image_Stage11_ImageFilename>images/expressionPedal/expres12.bmp</Image_Stage11_ImageFilename>" & vbCrLf
        txt += "<Image_Stage12_ImageFilename>images/expressionPedal/expres13.bmp</Image_Stage12_ImageFilename>" & vbCrLf
        txt += "<Image_Stage13_ImageFilename>images/expressionPedal/expres14.bmp</Image_Stage13_ImageFilename>" & vbCrLf
        txt += "<Image_Stage14_ImageFilename>images/expressionPedal/expres15.bmp</Image_Stage14_ImageFilename>" & vbCrLf
        txt += "<Image_Stage15_ImageFilename>images/expressionPedal/expres16.bmp</Image_Stage15_ImageFilename>" & vbCrLf
        txt += "<Text_HasText>N</Text_HasText>" & vbCrLf
        txt += "<Text_Font_FaceName></Text_Font_FaceName>" & vbCrLf
        txt += "<Text_Font_HeightPixels>0</Text_Font_HeightPixels>" & vbCrLf
        txt += "<Text_Font_WeightCode>0</Text_Font_WeightCode>" & vbCrLf
        txt += "<Text_Font_IsItalic>N</Text_Font_IsItalic>" & vbCrLf
        txt += "<Text_Font_IsUnderlined>N</Text_Font_IsUnderlined>" & vbCrLf
        txt += "<Text_RGBColour_Red>0</Text_RGBColour_Red>" & vbCrLf
        txt += "<Text_RGBColour_Green>0</Text_RGBColour_Green>" & vbCrLf
        txt += "<Text_RGBColour_Blue>0</Text_RGBColour_Blue>" & vbCrLf
        txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_LeftX>0</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_LeftX>" & vbCrLf
        txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_TopY>0</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_TopY>" & vbCrLf
        txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_RightX>0</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_RightX>" & vbCrLf
        txt += "<Text_BoundingBoxPosPixelsRelToTopLeftOfImage_Bottom>0</Text_BoundingBoxPosPixelsRelToTopLeftOfImage_Bottom>" & vbCrLf
        txt += "</customdisplaycontrolstyle>" & vbCrLf & vbCrLf
        Main.RtbCODF.Text += txt
    End Sub
    Public Sub make_rank(ByVal rk As Integer)
        Dim tuning As Integer = 0 ' par defaut
        Dim volume As Integer = 6 + rank(rk).vol36 ' par defaut
        Dim filter36 As Integer = 0 ' par defaut
        Dim filter96 As Integer = 0 ' par defaut
        Dim txt As String = ""
        With rank(rk)
            txt += "<rank>" & vbCrLf
            txt += "<!--" & .knobReloadCode & .knobNumber & "-->" & vbCrLf
            txt += "<!--" & .KnobName & "-->" & vbCrLf
            txt += "<RankID>" & .ID & "</RankID>" & vbCrLf
            txt += "<Name>" & .name & "</Name>" & vbCrLf
            txt += "<NumberOfPipes>" & .numberOfPipes & "</NumberOfPipes>" & vbCrLf
            txt += "<MIDINoteNumberOfFirstPipe>36</MIDINoteNumberOfFirstPipe>" & vbCrLf
            txt += "<IsPercussive>N</IsPercussive>" & vbCrLf
            txt += "<Samples_InstallationPackageID>001512</Samples_InstallationPackageID>" & vbCrLf
            ' JEUX MODIFIES (TRANSPOSES, FILTRES OU ATTENNUES)
            Select Case .folder
                Case "Manuals/Tierce_315" : tuning = -1200 : volume += -3 : filter96 = +1
                    txt += "<Samples_MainSampleFileFolder>Manuals/Tierce_135</Samples_MainSampleFileFolder>" & vbCrLf
                Case "Manuals/Quinte_513" : tuning = -1200 : volume += -3
                    txt += "<Samples_MainSampleFileFolder>Manuals/Nasard_223</Samples_MainSampleFileFolder>" & vbCrLf
                Case "Manuals/Quintaton_8" : tuning = +1200 : volume += -2
                    txt += "<Samples_MainSampleFileFolder>Manuals/Quintaton_16</Samples_MainSampleFileFolder>" & vbCrLf
                Case "Pedals/Basson_16" : tuning = -1200 : volume += +1 : filter36 = +2 : filter96 = -2
                    txt += "<Samples_MainSampleFileFolder>Manuals/Hautbois_8</Samples_MainSampleFileFolder>" & vbCrLf
                Case "Manuals/Septade_117" : volume += -6
                    txt += "<Samples_MainSampleFileFolder>" & .folder & "</Samples_MainSampleFileFolder>" & vbCrLf
                Case "Manuals/Piccolo_1" : volume += -7
                    txt += "<Samples_MainSampleFileFolder>" & .folder & "</Samples_MainSampleFileFolder>" & vbCrLf
                Case "Manuals/Fourniture_V" : tuning = +1200 : volume += -5 : filter96 = -3 : filter36 = -2
                    txt += "<Samples_MainSampleFileFolder>" & .folder & "</Samples_MainSampleFileFolder>" & vbCrLf

                Case Else ' JEUX NORMAUX
                    txt += "<Samples_MainSampleFileFolder>" & .folder & "</Samples_MainSampleFileFolder>" & vbCrLf
            End Select
            txt += "<Samples_SamplesAreProtected>N</Samples_SamplesAreProtected>" & vbCrLf
            txt += "<Samples_ReleaseSamplesStoredInSeparateFilesIfNotPercussive>n</Samples_ReleaseSamplesStoredInSeparateFilesIfNotPercussive>" & vbCrLf
            txt += "<Samples_RelSampleFileFolderIfRelSamplesStoredInSeparateFiles></Samples_RelSampleFileFolderIfRelSamplesStoredInSeparateFiles>" & vbCrLf
            txt += "<Samples_MIDINoteNumIncrementFromPipesToSamples>0</Samples_MIDINoteNumIncrementFromPipesToSamples>" & vbCrLf
            txt += "<Samples_SampleMIDINoteNumOfFirstAvailableSample>36</Samples_SampleMIDINoteNumOfFirstAvailableSample>" & vbCrLf
            txt += "<Samples_SampleMIDINoteNumOfLastAvailableSample>" & (36 + .numberOfPipes) & "</Samples_SampleMIDINoteNumOfLastAvailableSample>" & vbCrLf
            txt += "<Samples_MIDINoteNumIncBetweenSamples>1</Samples_MIDINoteNumIncBetweenSamples>" & vbCrLf
            txt += "<Samples_AssumeTunedToConcertPitchAndEqualTemp>Y</Samples_AssumeTunedToConcertPitchAndEqualTemp>" & vbCrLf
            txt += "<Samples_RankBasePitch64ftHarmNumIfAssumedTunedToConcertPitch>8</Samples_RankBasePitch64ftHarmNumIfAssumedTunedToConcertPitch>" & vbCrLf
            txt += "<Samples_AudioOutOptimalChannelFormatCode>1</Samples_AudioOutOptimalChannelFormatCode>" & vbCrLf
            txt += "<Samples_AudioOutOptimalSampleResolutionCode>3</Samples_AudioOutOptimalSampleResolutionCode>" & vbCrLf
            txt += "<Position_LayoutOfPipesOnWindchestCode>5</Position_LayoutOfPipesOnWindchestCode>" & vbCrLf
            txt += "<Position_RankDistanceFromListenerMetres>1</Position_RankDistanceFromListenerMetres>" & vbCrLf
            txt += "<WindSupply_DivisionCodeOfWindchest>" & .manual + 1 & "</WindSupply_DivisionCodeOfWindchest>" & vbCrLf
            txt += "<WindSupply_MaxFlowRandomisationPct>" & Rnd() * maxFlow & "</WindSupply_MaxFlowRandomisationPct>" & vbCrLf
            txt += "<Amp_PipeMIDINoteNum036_LevelAdjustDecibels>" & volume & "</Amp_PipeMIDINoteNum036_LevelAdjustDecibels>" & vbCrLf
            txt += "<Amp_PipeMIDINoteNum096_LevelAdjustDecibels>" & volume & "</Amp_PipeMIDINoteNum096_LevelAdjustDecibels>" & vbCrLf
            txt += "<Amp_PipeMIDINoteNum036_WindModelModDepthAdjustDecibels>0</Amp_PipeMIDINoteNum036_WindModelModDepthAdjustDecibels>" & vbCrLf
            txt += "<Amp_PipeMIDINoteNum096_WindModelModDepthAdjustDecibels>0</Amp_PipeMIDINoteNum096_WindModelModDepthAdjustDecibels>" & vbCrLf
            txt += "<Amp_PipeMIDINoteNum036_TremulantModDepthAdjustDecibels>0</Amp_PipeMIDINoteNum036_TremulantModDepthAdjustDecibels>" & vbCrLf
            txt += "<Amp_PipeMIDINoteNum096_TremulantModDepthAdjustDecibels>0</Amp_PipeMIDINoteNum096_TremulantModDepthAdjustDecibels>" & vbCrLf
            txt += "<Pitch_RankBaseOutputPitch64ftHarmonicNum>8</Pitch_RankBaseOutputPitch64ftHarmonicNum>" & vbCrLf
            txt += "<Pitch_MaxRandomTuningErrorHz>0.25</Pitch_MaxRandomTuningErrorHz>" & vbCrLf
            txt += "<Pitch_PipeMIDINoteNum036_TuningAdjustPercentSemitones>" & tuning & "</Pitch_PipeMIDINoteNum036_TuningAdjustPercentSemitones>" & vbCrLf
            txt += "<Pitch_PipeMIDINoteNum096_TuningAdjustPercentSemitones>" & tuning & "</Pitch_PipeMIDINoteNum096_TuningAdjustPercentSemitones>" & vbCrLf
            txt += "<Pitch_PipeMIDINoteNum036_WindModelModDepthAdjustPercent>95</Pitch_PipeMIDINoteNum036_WindModelModDepthAdjustPercent>" & vbCrLf
            txt += "<Pitch_PipeMIDINoteNum096_WindModelModDepthAdjustPercent>80</Pitch_PipeMIDINoteNum096_WindModelModDepthAdjustPercent>" & vbCrLf
            txt += "<Pitch_PipeMIDINoteNum036_TremulantModDepthAdjustPercent>100</Pitch_PipeMIDINoteNum036_TremulantModDepthAdjustPercent>" & vbCrLf
            txt += "<Pitch_PipeMIDINoteNum096_TremulantModDepthAdjustPercent>100</Pitch_PipeMIDINoteNum096_TremulantModDepthAdjustPercent>" & vbCrLf
            txt += "<HarmonicShaping_PipeMIDINoteNum036_3rdAndUpperHarmLvlAdjDb>" & filter36 & "</HarmonicShaping_PipeMIDINoteNum036_3rdAndUpperHarmLvlAdjDb>" & vbCrLf
            txt += "<HarmonicShaping_PipeMIDINoteNum096_3rdAndUpperHarmLvlAdjDb>" & filter96 & "</HarmonicShaping_PipeMIDINoteNum096_3rdAndUpperHarmLvlAdjDb>" & vbCrLf
            txt += "<HarmonicShaping_PipeMIDINoteNum036_WindModelModDepthAdjDb>0</HarmonicShaping_PipeMIDINoteNum036_WindModelModDepthAdjDb>" & vbCrLf
            txt += "<HarmonicShaping_PipeMIDINoteNum096_WindModelModDepthAdjDb>0.5</HarmonicShaping_PipeMIDINoteNum096_WindModelModDepthAdjDb>" & vbCrLf
            txt += "<HarmonicShaping_PipeMIDINoteNum036_TremulantModDepthAdjDb>0</HarmonicShaping_PipeMIDINoteNum036_TremulantModDepthAdjDb>" & vbCrLf
            txt += "<HarmonicShaping_PipeMIDINoteNum096_TremulantModDepthAdjDb>0</HarmonicShaping_PipeMIDINoteNum096_TremulantModDepthAdjDb>" & vbCrLf
            If .isEnclosed Then
                txt += "<Encl_IsEnclosed>Y</Encl_IsEnclosed>" & vbCrLf
                txt += "<Encl_EnclosureCode>" & .enclosureCode & "</Encl_EnclosureCode>" & vbCrLf
                txt += "<Encl_FiltParamWhenClsdForPipeMIDINoteNum036_OverallAttnDb>" & Int(Rnd() * 4) & "</Encl_FiltParamWhenClsdForPipeMIDINoteNum036_OverallAttnDb>" & vbCrLf
                txt += "<Encl_FiltParamWhenClsdForPipeMIDINoteNum036_MaxFreqHz>200</Encl_FiltParamWhenClsdForPipeMIDINoteNum036_MaxFreqHz>" & vbCrLf
                txt += "<Encl_FiltParamWhenClsdForPipeMIDINoteNum036_MinFreqHz>1000</Encl_FiltParamWhenClsdForPipeMIDINoteNum036_MinFreqHz>" & vbCrLf
                txt += "<Encl_FiltParamWhenClsdForPipeMIDINoteNum036_ExtraAttnAtMinDb>" & 4 + Int(Rnd() * 4) & "</Encl_FiltParamWhenClsdForPipeMIDINoteNum036_ExtraAttnAtMinDb>" & vbCrLf
                txt += "<Encl_FiltParamWhenOpenForPipeMIDINoteNum036_MaxFreqHz>100</Encl_FiltParamWhenOpenForPipeMIDINoteNum036_MaxFreqHz>" & vbCrLf
                txt += "<Encl_FiltParamWhenOpenForPipeMIDINoteNum036_MinFreqHz>8000</Encl_FiltParamWhenOpenForPipeMIDINoteNum036_MinFreqHz>" & vbCrLf
                txt += "<Encl_FiltParamWhenClsdForPipeMIDINoteNum096_OverallAttnDb>" & 1 + Int(Rnd() * 4) & "</Encl_FiltParamWhenClsdForPipeMIDINoteNum096_OverallAttnDb>" & vbCrLf
                txt += "<Encl_FiltParamWhenClsdForPipeMIDINoteNum096_MaxFreqHz>200</Encl_FiltParamWhenClsdForPipeMIDINoteNum096_MaxFreqHz>" & vbCrLf
                txt += "<Encl_FiltParamWhenClsdForPipeMIDINoteNum096_MinFreqHz>1000</Encl_FiltParamWhenClsdForPipeMIDINoteNum096_MinFreqHz>" & vbCrLf
                txt += "<Encl_FiltParamWhenClsdForPipeMIDINoteNum096_ExtraAttnAtMinDb>" & 5 + Int(Rnd() * 6) & "</Encl_FiltParamWhenClsdForPipeMIDINoteNum096_ExtraAttnAtMinDb>" & vbCrLf
                txt += "<Encl_FiltParamWhenOpenForPipeMIDINoteNum096_MaxFreqHz>100</Encl_FiltParamWhenOpenForPipeMIDINoteNum096_MaxFreqHz>" & vbCrLf
                txt += "<Encl_FiltParamWhenOpenForPipeMIDINoteNum096_MinFreqHz>8000</Encl_FiltParamWhenOpenForPipeMIDINoteNum096_MinFreqHz>" & vbCrLf
                txt += "<Encl_MIDINoteNumOfFirstEnclosedPipeIfWholeRankNotEnclosed>0</Encl_MIDINoteNumOfFirstEnclosedPipeIfWholeRankNotEnclosed>" & vbCrLf
                txt += "<Encl_MIDINoteNumOfLastEnclosedPipeIfWholeRankNotEnclosed>0</Encl_MIDINoteNumOfLastEnclosedPipeIfWholeRankNotEnclosed>" & vbCrLf

            Else
                txt += "<Encl_IsEnclosed>N</Encl_IsEnclosed>" & vbCrLf
                txt += "<Encl_EnclosureCode></Encl_EnclosureCode>" & vbCrLf
                txt += "<Encl_FiltParamWhenClsdForPipeMIDINoteNum036_OverallAttnDb>0</Encl_FiltParamWhenClsdForPipeMIDINoteNum036_OverallAttnDb>" & vbCrLf
                txt += "<Encl_FiltParamWhenClsdForPipeMIDINoteNum036_MaxFreqHz>0</Encl_FiltParamWhenClsdForPipeMIDINoteNum036_MaxFreqHz>" & vbCrLf
                txt += "<Encl_FiltParamWhenClsdForPipeMIDINoteNum036_MinFreqHz>0</Encl_FiltParamWhenClsdForPipeMIDINoteNum036_MinFreqHz>" & vbCrLf
                txt += "<Encl_FiltParamWhenClsdForPipeMIDINoteNum036_ExtraAttnAtMinDb>0</Encl_FiltParamWhenClsdForPipeMIDINoteNum036_ExtraAttnAtMinDb>" & vbCrLf
                txt += "<Encl_FiltParamWhenOpenForPipeMIDINoteNum036_MaxFreqHz>0</Encl_FiltParamWhenOpenForPipeMIDINoteNum036_MaxFreqHz>" & vbCrLf
                txt += "<Encl_FiltParamWhenOpenForPipeMIDINoteNum036_MinFreqHz>0</Encl_FiltParamWhenOpenForPipeMIDINoteNum036_MinFreqHz>" & vbCrLf
                txt += "<Encl_FiltParamWhenClsdForPipeMIDINoteNum096_OverallAttnDb>0</Encl_FiltParamWhenClsdForPipeMIDINoteNum096_OverallAttnDb>" & vbCrLf
                txt += "<Encl_FiltParamWhenClsdForPipeMIDINoteNum096_MaxFreqHz>0</Encl_FiltParamWhenClsdForPipeMIDINoteNum096_MaxFreqHz>" & vbCrLf
                txt += "<Encl_FiltParamWhenClsdForPipeMIDINoteNum096_MinFreqHz>0</Encl_FiltParamWhenClsdForPipeMIDINoteNum096_MinFreqHz>" & vbCrLf
                txt += "<Encl_FiltParamWhenClsdForPipeMIDINoteNum096_ExtraAttnAtMinDb>0</Encl_FiltParamWhenClsdForPipeMIDINoteNum096_ExtraAttnAtMinDb>" & vbCrLf
                txt += "<Encl_FiltParamWhenOpenForPipeMIDINoteNum096_MaxFreqHz>0</Encl_FiltParamWhenOpenForPipeMIDINoteNum096_MaxFreqHz>" & vbCrLf
                txt += "<Encl_FiltParamWhenOpenForPipeMIDINoteNum096_MinFreqHz>0</Encl_FiltParamWhenOpenForPipeMIDINoteNum096_MinFreqHz>" & vbCrLf
                txt += "<Encl_MIDINoteNumOfFirstEnclosedPipeIfWholeRankNotEnclosed>0</Encl_MIDINoteNumOfFirstEnclosedPipeIfWholeRankNotEnclosed>" & vbCrLf
                txt += "<Encl_MIDINoteNumOfLastEnclosedPipeIfWholeRankNotEnclosed>0</Encl_MIDINoteNumOfLastEnclosedPipeIfWholeRankNotEnclosed>" & vbCrLf
            End If
            If .hasTremulant = True Then
                txt += "<Trem_HasTremulant>Y</Trem_HasTremulant>" & vbCrLf
                txt += "<Trem_TremulantCode>" & .tremulantCode & "</Trem_TremulantCode>" & vbCrLf
                txt += "<Trem_SampleInstallationPackageID>1512</Trem_SampleInstallationPackageID>" & vbCrLf
                txt += "<Trem_WaveformSampleFileFolder>tremulant001</Trem_WaveformSampleFileFolder>" & vbCrLf
                txt += "<Trem_SamplesAreProtected>N</Trem_SamplesAreProtected>" & vbCrLf
                txt += "<Trem_MIDINoteNumIncrementFromPipesToTremSamples>0</Trem_MIDINoteNumIncrementFromPipesToTremSamples>" & vbCrLf
                txt += "<Trem_SampleMIDINoteNumberOfFirstAvailableTremSample>0</Trem_SampleMIDINoteNumberOfFirstAvailableTremSample>" & vbCrLf
                txt += "<Trem_SampleMIDINoteNumberOfLastAvailableTremSample>0</Trem_SampleMIDINoteNumberOfLastAvailableTremSample>" & vbCrLf
                txt += "<Trem_MIDINoteNumIncBetweenTremSamples>0</Trem_MIDINoteNumIncBetweenTremSamples>" & vbCrLf
                txt += "<Trem_MaxDepthRandomisationPct>10</Trem_MaxDepthRandomisationPct>" & vbCrLf
                txt += "<Trem_MIDINoteNumOfFirstTremmedPipeIfWholeRankNotTremmed>0</Trem_MIDINoteNumOfFirstTremmedPipeIfWholeRankNotTremmed>" & vbCrLf
                txt += "<Trem_MIDINoteNumOfLastTremmedPipeIfWholeRankNotTremmed>0</Trem_MIDINoteNumOfLastTremmedPipeIfWholeRankNotTremmed>" & vbCrLf
            Else
                txt += "<Trem_HasTremulant>N</Trem_HasTremulant>" & vbCrLf
                txt += "<Trem_TremulantCode></Trem_TremulantCode>" & vbCrLf
                txt += "<Trem_SampleInstallationPackageID></Trem_SampleInstallationPackageID>" & vbCrLf
                txt += "<Trem_WaveformSampleFileFolder></Trem_WaveformSampleFileFolder>" & vbCrLf
                txt += "<Trem_SamplesAreProtected>N</Trem_SamplesAreProtected>" & vbCrLf
                txt += "<Trem_MIDINoteNumIncrementFromPipesToTremSamples>0</Trem_MIDINoteNumIncrementFromPipesToTremSamples>" & vbCrLf
                txt += "<Trem_SampleMIDINoteNumberOfFirstAvailableTremSample>0</Trem_SampleMIDINoteNumberOfFirstAvailableTremSample>" & vbCrLf
                txt += "<Trem_SampleMIDINoteNumberOfLastAvailableTremSample>0</Trem_SampleMIDINoteNumberOfLastAvailableTremSample>" & vbCrLf
                txt += "<Trem_MIDINoteNumIncBetweenTremSamples>0</Trem_MIDINoteNumIncBetweenTremSamples>" & vbCrLf
                txt += "<Trem_MaxDepthRandomisationPct>0</Trem_MaxDepthRandomisationPct>" & vbCrLf
                txt += "<Trem_MIDINoteNumOfFirstTremmedPipeIfWholeRankNotTremmed>0</Trem_MIDINoteNumOfFirstTremmedPipeIfWholeRankNotTremmed>" & vbCrLf
                txt += "<Trem_MIDINoteNumOfLastTremmedPipeIfWholeRankNotTremmed>0</Trem_MIDINoteNumOfLastTremmedPipeIfWholeRankNotTremmed>" & vbCrLf
            End If
            txt += "<ReverbTailTruncation_ModeCode>" & reverbMode & "</ReverbTailTruncation_ModeCode>" & vbCrLf
            txt += "<ReverbTailTruncation_DecayLengthAsMsForMiddleCOn8FtStop>" & reverbTime & "</ReverbTailTruncation_DecayLengthAsMsForMiddleCOn8FtStop>" & vbCrLf
            txt += "</rank>" & vbCrLf
        End With
        txt += "" & vbCrLf
        Main.RtbCODF.Text += txt
    End Sub
    Public Sub make_stoprank(ByVal sr As Integer)
        Dim txt As String = ""
        With stopRank(sr)
            txt += "<stoprank>" & vbCrLf
            txt += "<Name>" & .name & "</Name>" & vbCrLf
            txt += "<StopCode>" & .stopCode & "</StopCode>" & vbCrLf
            txt += "<RankTypeCode>1</RankTypeCode>" & vbCrLf
            txt += "<RankID>" & .rankID & "</RankID>" & vbCrLf
            txt += "<ExternalRankID></ExternalRankID>" & vbCrLf
            txt += "<MIDINoteNumIncrementFromDivisionToRank>0</MIDINoteNumIncrementFromDivisionToRank>" & vbCrLf
            txt += "<MIDINoteNumOfFirstMappedDivisionKey>36</MIDINoteNumOfFirstMappedDivisionKey>" & vbCrLf
            txt += "<NumberOfMappedDivisionKeys>" & .numberOfKeys & "</NumberOfMappedDivisionKeys>" & vbCrLf
            txt += "<ActionTypeCode>1</ActionTypeCode>" & vbCrLf
            txt += "<ActionEffectCode>1</ActionEffectCode>" & vbCrLf
            txt += "<PipeMIDINoteNum036_PizzOrReitPeriodMs>0</PipeMIDINoteNum036_PizzOrReitPeriodMs>" & vbCrLf
            txt += "<PipeMIDINoteNum096_PizzOrReitPeriodMs>0</PipeMIDINoteNum096_PizzOrReitPeriodMs>" & vbCrLf
            txt += "<StopCodeToSwitchToAlternateRank></StopCodeToSwitchToAlternateRank>" & vbCrLf
            txt += "<RetriggerNotesWhenSwitchingBetweenNormalAndAlternateRanks>N</RetriggerNotesWhenSwitchingBetweenNormalAndAlternateRanks>" & vbCrLf
            txt += "<AlternateRankID></AlternateRankID>" & vbCrLf
            txt += "<AlternateExternalRankID></AlternateExternalRankID>" & vbCrLf
            txt += "</stoprank>" & vbCrLf
        End With
        txt += "" & vbCrLf
        Main.RtbCODF.Text += txt
    End Sub
    Public Sub make_Stop(ByVal stp As Integer)
        Dim txt As String = ""
        With stopknob(stp)
            txt += "<stop>" & vbCrLf
            txt += "<StopCode>" & .stopCode & "</StopCode>" & vbCrLf
            txt += "<Name>" & .name & "</Name>" & vbCrLf
            txt += "<DisplayName>" & .displayname & "</DisplayName>" & vbCrLf
            txt += "<DefaultToEngaged>N</DefaultToEngaged>" & vbCrLf
            txt += "<SwitchIsLatching>Y</SwitchIsLatching>" & vbCrLf
            txt += "<StoreInDivisionalCombinations>Y</StoreInDivisionalCombinations>" & vbCrLf
            txt += "<StoreInGeneralCombinations>Y</StoreInGeneralCombinations>" & vbCrLf
            txt += "<AccessibleForMIDIInputAndOutput>Y</AccessibleForMIDIInputAndOutput>" & vbCrLf
            txt += "<OverrideSourceDivisionToSpecifiedDivisionCode></OverrideSourceDivisionToSpecifiedDivisionCode>" & vbCrLf
            txt += "<StopCodeFromWhichToCopyState></StopCodeFromWhichToCopyState>" & vbCrLf
            txt += "<PercussiveEngagingSoundEffect_RankID></PercussiveEngagingSoundEffect_RankID>" & vbCrLf
            txt += "<PercussiveEngagingSoundEffect_MIDINoteNumber>0</PercussiveEngagingSoundEffect_MIDINoteNumber>" & vbCrLf
            txt += "<PercussiveDisengagingSoundEffect_RankID></PercussiveDisengagingSoundEffect_RankID>" & vbCrLf
            txt += "<PercussiveDisengagingSoundEffect_MIDINoteNumber>0</PercussiveDisengagingSoundEffect_MIDINoteNumber>" & vbCrLf
            txt += "<SustainingSoundEffect_RankID></SustainingSoundEffect_RankID>" & vbCrLf
            txt += "<SustainingSoundEffect_MIDINoteNumber>0</SustainingSoundEffect_MIDINoteNumber>" & vbCrLf
            txt += "<CustomDisplay1_DisplayPageID>1</CustomDisplay1_DisplayPageID>" & vbCrLf
            txt += "<CustomDisplay1_Text>" & .text & "</CustomDisplay1_Text>" & vbCrLf
            txt += "<CustomDisplay1_ControlStyleID>" & .styleID1 & "</CustomDisplay1_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay1_PosPixels_X>" & .x1 & "</CustomDisplay1_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay1_PosPixels_Y>" & .y1 & "</CustomDisplay1_PosPixels_Y>" & vbCrLf
            txt += "<CustomDisplay2_DisplayPageID>" & .jamb & "</CustomDisplay2_DisplayPageID>" & vbCrLf
            txt += "<CustomDisplay2_Text>" & .text & "</CustomDisplay2_Text>" & vbCrLf
            txt += "<CustomDisplay2_ControlStyleID>" & .styleID2 & "</CustomDisplay2_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay2_PosPixels_X>" & .x2 & "</CustomDisplay2_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay2_PosPixels_Y>" & .y2 & "</CustomDisplay2_PosPixels_Y>" & vbCrLf
            txt += "</stop>" & vbCrLf
        End With
        txt += "" & vbCrLf
        Main.RtbCODF.Text += txt
    End Sub
    Public Sub make_tremulant(ByVal tr As Integer)
        Dim txt As String = ""
        With tremulant(tr)
            txt += "<tremulant>" & vbCrLf
            txt += "<TremulantCode>" & .tremulantCode & "</TremulantCode>" & vbCrLf
            txt += "<Name>Tremulant: " & .manual & "</Name>" & vbCrLf
            txt += "<CustomDisplay1_DisplayPageID>1</CustomDisplay1_DisplayPageID>" & vbCrLf
            txt += "<CustomDisplay1_Text>TREMULANT</CustomDisplay1_Text>" & vbCrLf
            txt += "<CustomDisplay1_ControlStyleID>" & .styleID1 & "</CustomDisplay1_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay1_PosPixels_X>" & .x1 & "</CustomDisplay1_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay1_PosPixels_Y>" & .y1 & "</CustomDisplay1_PosPixels_Y>" & vbCrLf
            txt += "<CustomDisplay1_AlternateLayout1_ControlStyleID></CustomDisplay1_AlternateLayout1_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay1_AlternateLayout1_PosPixels_X>0</CustomDisplay1_AlternateLayout1_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay1_AlternateLayout1_PosPixels_Y>0</CustomDisplay1_AlternateLayout1_PosPixels_Y>" & vbCrLf
            txt += "<CustomDisplay2_DisplayPageID>" & .jamb & "</CustomDisplay2_DisplayPageID>" & vbCrLf
            txt += "<CustomDisplay2_Text>TREMULANT</CustomDisplay2_Text>" & vbCrLf
            txt += "<CustomDisplay2_ControlStyleID>" & .styleID2 & "</CustomDisplay2_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay2_PosPixels_X>" & .x2 & "</CustomDisplay2_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay2_PosPixels_Y>" & .y2 & "</CustomDisplay2_PosPixels_Y>" & vbCrLf
            txt += "<CustomDisplay2_AlternateLayout1_ControlStyleID></CustomDisplay2_AlternateLayout1_ControlStyleID>" & vbCrLf
            txt += "<CustomDisplay2_AlternateLayout1_PosPixels_X>0</CustomDisplay2_AlternateLayout1_PosPixels_X>" & vbCrLf
            txt += "<CustomDisplay2_AlternateLayout1_PosPixels_Y>0</CustomDisplay2_AlternateLayout1_PosPixels_Y>" & vbCrLf
            txt += "<DisplayName>TREMULANT</DisplayName>" & vbCrLf
            txt += "<DefaultToEngaged>N</DefaultToEngaged>" & vbCrLf
            txt += "<SwitchIsLatching>Y</SwitchIsLatching>" & vbCrLf
            txt += "<StoreInDivisionalCombinations>Y</StoreInDivisionalCombinations>" & vbCrLf
            txt += "<StoreInGeneralCombinations>Y</StoreInGeneralCombinations>" & vbCrLf
            txt += "<AccessibleForMIDIInputAndOutput>Y</AccessibleForMIDIInputAndOutput>" & vbCrLf
            txt += "<PercussiveEngagingSoundEffect_RankID></PercussiveEngagingSoundEffect_RankID>" & vbCrLf
            txt += "<PercussiveEngagingSoundEffect_MIDINoteNumber>0</PercussiveEngagingSoundEffect_MIDINoteNumber>" & vbCrLf
            txt += "<PercussiveDisengagingSoundEffect_RankID></PercussiveDisengagingSoundEffect_RankID>" & vbCrLf
            txt += "<PercussiveDisengagingSoundEffect_MIDINoteNumber>0</PercussiveDisengagingSoundEffect_MIDINoteNumber>" & vbCrLf
            txt += "<SustainingSoundEffect_RankID></SustainingSoundEffect_RankID>" & vbCrLf
            txt += "<SustainingSoundEffect_MIDINoteNumber>0</SustainingSoundEffect_MIDINoteNumber>" & vbCrLf
            txt += "<TremulantCodeFromWhichToCopyState></TremulantCodeFromWhichToCopyState>" & vbCrLf
            txt += "<FrequencyHz>" & tremulantFrequency & "</FrequencyHz>" & vbCrLf
            txt += "<MaxFrequencyRandomisationPct>15</MaxFrequencyRandomisationPct>" & vbCrLf
            txt += "<MaxDepthRandomisationPct>15</MaxDepthRandomisationPct>" & vbCrLf
        End With
        txt += "</tremulant>" & vbCrLf & vbCrLf
        Main.RtbCODF.Text += txt
    End Sub
End Module
